# -*- coding: utf-8 -*-
"""
Created on Sun Jul 21 22:13:21 2019

@author: qchat
"""


import os
import shutil






def checkForbiddenCharacters(value):
    try : 
        assert '/' not in value
        assert '<' not in value
        assert '>' not in value
        assert ':' not in value
        assert '?' not in value
        assert '!' not in value
        assert '\\' not in value
        assert '*' not in value
        assert '|' not in value
        return True
    except :
        return False  
    
    
    
    
class Recorder :
    
    def __init__(self,name,customPath=None,verbose=True):
        
        if isinstance(name,str) is False or checkForbiddenCharacters(name) is False:
            raise ValueError(f'The name "{name}" is not valid')           
            		
        print(f'Starting Recorder with name "{name}"')
                
        self.verbose = verbose 
        
        self.name = name
        self.path = None
        
        self.var = {}
        self.varList= []
        self.started = False
        self.count = 0
        self.mainFile = None
        
        
        if customPath is None :
            self.setPath(os.getcwd())
        else :
            self.createFolder(customPath)
            self.setPath(customPath)
        
    #--------------------------------------------------------------------------
    # Path
    #--------------------------------------------------------------------------
        
    def createFolder(self,path):
        if os.path.exists(path) is False :
            parentFolder = os.path.realpath(os.path.dirname(path))
            if os.path.exists(parentFolder) is False :
                self.createFolder(parentFolder)
            os.mkdir(path)
                
    
    def getPath(self):
        return self.path
    
    def setPath(self,folderPath):
        if self.started is False :
            
            if os.path.exists(folderPath) is False :
                raise ValueError(f'The path "{os.path.dirname(folderPath)}" does not exists')
    
            path = os.path.join(folderPath,self.name)
            compt = 0
            while True :
                if os.path.exists(path):
                    compt += 1
                    path = os.path.join(folderPath,self.name+'_'+str(compt))
                else :
                    break
            self.path = path
            print(f'Recorder "{self.name}" will save its data at "{self.path}"')
            
        else : 
            raise ValueError('You cannot change the path after having started to save your data')
        
        
        
    #--------------------------------------------------------------------------
    # Variables
    #--------------------------------------------------------------------------    
    
    def setValue(self,varName,value):
        if isinstance(varName,str) is False :
            raise ValueError(f'"{varName}" is not a valid name for a variable')
        if self.started and varName not in self.getVariableList() :
            raise ValueError('You cannot add a new variable after having started to save your data')
        if varName not in self.getVariableList():
            self.varList.append(varName)
        self.var[varName] = value
        
        
    def getValue(self,varName):
        if varName not in self.getVariableList():
            raise ValueError(f'Variable unknown "{varName}"')
        return self.var[varName]
        
        
    def getVariableList(self):
        return tuple(self.varList)

    
    def getValueType(self,varName):
        if varName not in self.getVariableList():
            raise ValueError(f'Variable unknown "{varName}"')
        return type(self.var[varName])
        
    
    #--------------------------------------------------------------------------
    # Data file
    #--------------------------------------------------------------------------     

    def initialize(self):
        
        import pandas as pd
        self.started = True
        
        os.mkdir(self.path)
        
        # Sauvegarde script
        try :
            scriptPath = os.path.realpath(__file__)
            shutil.copyfile(scriptPath,os.path.join(self.path,'script_backup.py'))
        except :
            pass
        
        # Création datafile        
        headerLine = '# MeasID - Variable'
        for varName in self.getVariableList() :
            if self.getValueType(varName) != pd.DataFrame :
                headerLine += f',{varName}'
            else :
                os.mkdir(os.path.join(self.path,varName.replace(' ','_')))
                        
        self.mainFile = open(os.path.join(self.path,'data.txt'), 'w')
        self.mainFile.write(headerLine+'\n')
        
        
    def save(self):
        
        import pandas as pd
        
        if self.started is False :
            self.initialize()
            
        self.count += 1
        
        valueLine = f'{self.count}'

        for varName in self.getVariableList() :
            if self.getValueType(varName) != pd.DataFrame :
                valueLine += f',{self.getValue(varName)}'
            else :
                varValue = self.getValue(varName)
                varValue.to_csv(os.path.join(self.path,varName.replace(' ','_'),f'{self.count}.txt'),index=False)

                        
        self.mainFile.write(valueLine+'\n')
        
        if self.verbose is True :
            print()
            print(f'Recorder "{self.name}" saving data point #{self.count} :')
            for varName in self.getVariableList() :
                print(f' - {varName} = {self.getValue(varName)}')
            print()
        

    def close(self):
        if self.started is True :
            self.mainFile.close()
        print(f'Recorder "{self.name}" closed')
    
            
            
            
class Recorder_V2 :
    
    def __init__(self,name,customPath=None,verbose=True):
        
        import pandas as pd
        
        if isinstance(name,str) is False or checkForbiddenCharacters(name) is False:
            raise ValueError(f'The name "{name}" is not valid')           
            
        print(f'Starting Recorder with name "{name}"')
            		
        self.verbose = verbose 
        
        self.name = name
        self.path = None
        
        self.var = {}
        self.varNameHistory = []
        self.dataframe = pd.DataFrame()
        self.started = False
        self.count = 0
        
        
        if customPath is None :
            self.setPath(os.getcwd())
        else :
            self.createFolder(customPath)
            self.setPath(customPath)
        
    #--------------------------------------------------------------------------
    # Path
    #--------------------------------------------------------------------------
        
    def createFolder(self,path):
        if os.path.exists(path) is False :
            parentFolder = os.path.realpath(os.path.dirname(path))
            if os.path.exists(parentFolder) is False :
                self.createFolder(parentFolder)
            os.mkdir(path)
                
    
    def getPath(self):
        return self.path
    
    def setPath(self,folderPath):
        if self.started is False :
            
            if os.path.exists(folderPath) is False :
                raise ValueError(f'The path "{os.path.dirname(folderPath)}" does not exists')
    
            path = os.path.join(folderPath,self.name)
            compt = 0
            while True :
                if os.path.exists(path):
                    compt += 1
                    path = os.path.join(folderPath,self.name+'_'+str(compt))
                else :
                    break
            self.path = path
            print(f'Recorder "{self.name}" will save its data at "{self.path}"')
            
        else : 
            raise ValueError('You cannot change the path after having started to save your data')
        
        
        
    #--------------------------------------------------------------------------
    # Variables
    #--------------------------------------------------------------------------    
    
    def getVariableList(self):
        return tuple(self.var.keys())
    
    def setValue(self,varName,value):
        if isinstance(varName,str) is False :
            raise ValueError(f'"{varName}" is not a valid name for a variable')
        if self.started and varName not in self.getVariableList() :
            raise ValueError('You cannot add a new variable after having started to save your data')
        assert type(value) in [str, int, float, bool]
        if varName not in self.getVariableList() :
            self.varNameHistory.append(varName)
        self.var[varName] = value
        
    def getValue(self,varName):
        if varName not in self.getVariableList():
            raise ValueError(f'Variable unknown "{varName}"')
        return self.var[varName]
        

    def getValueType(self,varName):
        if varName not in self.getVariableList():
            raise ValueError(f'Variable unknown "{varName}"')
        return type(self.var[varName])
        
    
    #--------------------------------------------------------------------------
    # Data file
    #--------------------------------------------------------------------------     

    def initialize(self):
        self.started = True
        
        os.mkdir(self.path)        

        
        
    def save(self):
        
        if self.started is False :
            self.initialize()
        
        # Ajout du set de variable à la fin du dataframe
        self.dataframe = self.dataframe.append(self.var, ignore_index=True)

        # Copie de la dernière ligne dans le fichier
        filePath = os.path.join(self.getPath(),'data.csv')
        if len(self.dataframe) == 1 :
            self.dataframe=self.dataframe[self.varNameHistory]
            self.dataframe.iloc[[-1]].to_csv(filePath,sep=';')
        else :
            self.dataframe.iloc[[-1]].to_csv(filePath,sep=';', mode='a', header=False)
            
        # Log
        if self.verbose is True :
            print()
            print(f'Recorder "{self.name}" saving data point #{self.count} :')
            print(self.dataframe.iloc[-1])
        

    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Function generator'

class Driver_parser():
    def __init__(self, Instance, name, **kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}

----------------  Examples:  ----------------

usage:    autolab driver [options] args
            
    autolab driver -D {self.name} -A GPIB::7::INSTR -C VISA -a 0.5 -f 10e6
    load {self.name} driver using VISA communication protocol with address TCPIP... and set the amplitude to 0.5V and frequency to 10MHz.
    
    autolab driver -D nickname -a 0.5 -f  10e6
    same as before but using the device nickname as defined in local_config.ini
            """
        return usage
    
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        parser.add_argument("-e", "--enable", action='store_true', dest="enable", default=None, help="Turn the RF output on." )
        parser.add_argument("-d", "--disable", action='store_true', dest="disable", default=None, help="Turn the RF output off." )
        parser.add_argument("-a", "--amplitude", type=str, dest="amplitude", default=None, help="Set the amplitude." )
        parser.add_argument("-f", "--frequency", type=str, dest="frequency", default=None, help="Set the frequency." )
        
        return parser

    def do_something(self,args):
        if args.amplitude:
            getattr(self.Instance,'set_rfamp')(args.amplitude)
        if args.frequency:
            getattr(self.Instance,'set_frequency')(args.frequency)
        if args.enable:
            getattr(self.Instance,'RFenable')()
        if args.disable:
            getattr(self.Instance,'RFdisable')()


    def exit(self):
        #self.Instance.close()
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Optical shutter'

class Driver_parser():
    def __init__(self, Instance, name, **kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}

----------------  Examples:  ----------------

usage:    autolab driver [options] args
        
    autolab driver -D {self.name} -A ASRL::2::INSTR -C VISA -m some_methods
    Execute some_methods of the driver. A list of available methods is present at the top of this help.
    
    autolab driver -D nickname -m some_methods
    Same as before using the nickname defined in local_config.ini
            """
        return usage
    
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        #parser.add_argument("-c", "--channels", type=str, dest="channels", default=None, help="Set the traces to act on/acquire from." )
        #parser.add_argument("-o", "--filename", type=str, dest="filename", default=None, help="Set the name of the output file" )
        #parser.add_argument("-F", "--force",action="store_true", dest="force", default=None, help="Allows overwriting file" )
        #parser.add_argument("-t", "--trigger", dest="trigger",action="store_true", help="Trigger the scope once" )
        
        return parser
    
    def do_something(self,args):
        #if args.filename:
            ##getattr(self.Instance,'get_data_traces')(traces=args.channels,single=args.trigger)
            #getattr(self.Instance,'get_data_traces')(traces=args.channels)
            #getattr(self.Instance,'save_data_traces')(filename=args.filename,traces=args.channels,FORCE=args.force)
        pass

    def exit(self):
        self.Instance.close()



# -*- coding: utf-8 -*-
"""
Created on Fri Sep 20 22:08:29 2019

@author: qchat
"""
from PyQt5 import QtCore, QtWidgets, uic
import os
import queue

from autolab import paths

from .data import DataManager
from .figure import FigureManager
from .monitor import MonitorManager

class Monitor(QtWidgets.QMainWindow):
        
    def __init__(self,item):
        
        self.item = item
        self.variable = item.variable
        
        # Configuration of the window
        QtWidgets.QMainWindow.__init__(self)
        ui_path = os.path.join(os.path.dirname(__file__),'interface.ui')
        uic.loadUi(ui_path,self)
        self.setWindowTitle(f"AUTOLAB Monitor : Variable {self.variable.name}")
        
        # Queue
        self.queue = queue.Queue()
        self.timer = QtCore.QTimer(self)
        self.timer.setInterval(33) #30fps
        self.timer.timeout.connect(self.sync)
        
        # Window length
        self.windowLength_lineEdit.setText('10')
        self.windowLength_lineEdit.returnPressed.connect(self.windowLengthChanged)
        self.windowLength_lineEdit.textEdited.connect(lambda : self.setLineEditBackground(self.windowLength_lineEdit,'edited'))
        self.setLineEditBackground(self.windowLength_lineEdit,'synced')

        # Delay
        self.delay_lineEdit.setText('0.01')
        self.delay_lineEdit.returnPressed.connect(self.delayChanged)
        self.delay_lineEdit.textEdited.connect(lambda : self.setLineEditBackground(self.delay_lineEdit,'edited'))
        self.setLineEditBackground(self.delay_lineEdit,'synced')
        
        # Pause
        self.pauseButton.clicked.connect(self.pauseButtonClicked)
        
        # Save
        self.saveButton.clicked.connect(self.saveButtonClicked)
        
        # Managers
        self.dataManager = DataManager(self)
        self.figureManager = FigureManager(self)
        self.monitorManager = MonitorManager(self)
        
        # Start
        self.windowLengthChanged()
        self.delayChanged()
        self.monitorManager.start()
        self.timer.start()
        
    
    def setLineEditBackground(self,obj,state):
        
        if state == 'synced' :
            color='#D2FFD2' # vert
        if state == 'edited' :
            color='#FFE5AE' # orange
            
        obj.setStyleSheet("QLineEdit:enabled {background-color: %s; font-size: 9pt}"%color)
        
        
        
    def sync(self):
        
        """ This function updates the data and than the figure. 
        Function called by the time """
                
        # Empty the queue
        count = 0
        while not self.queue.empty():
            self.dataManager.addPoint(self.queue.get())
            count += 1
        
        # Upload the plot if new data available
        if count > 0 :
            xlist,ylist = self.dataManager.getData()
            self.figureManager.update(xlist,ylist)
        
    
    
    def pauseButtonClicked(self):
        
        """ This function pause or resume the monitoring """
        
        if self.monitorManager.isPaused() is False :
            self.timer.stop()
            self.pauseButton.setText('Resume')
            self.monitorManager.pause()
        else :
            self.timer.start()
            self.pauseButton.setText('Pause')
            self.monitorManager.resume()
            
            
        
    def saveButtonClicked(self):
        
        """ This function is called when the SAVE button is pressed, and launch the procedure 
        to save both the data and the figure """
        
        # Make sure the monitoring is paused
        if self.monitorManager.isPaused() is False :
            self.pauseButtonClicked()
        
        # Ask the path of the output folder
        path = str(QtWidgets.QFileDialog.getExistingDirectory(self, "Select Directory",paths.USER_LAST_CUSTOM_FOLDER))
        
        # Save the given path for future, the data and the figure if the path provided is valid
        if path != '' :
            
            paths.USER_LAST_CUSTOM_FOLDER = path
            self.statusBar.showMessage('Saving data...',5000)
            
            try : 
                self.dataManager.save(path)
                self.figureManager.save(path)
                self.statusBar.showMessage(f'Data successfully saved in {path}.',5000)
            except :
                self.statusBar.showMessage('An error occured while saving data !',10000)
            

            
    def closeEvent(self,event):
        
        """ This function does some steps before the window is really killed """
        
        self.monitorManager.close()
        self.timer.stop()
        self.item.clearMonitor()
        
        
        

    def windowLengthChanged(self):
        
        """ This function start the update of the window length in the data manager
        when a changed has been detected """
        
        # Send the new value
        try : 
            value = float(self.windowLength_lineEdit.text())
            assert value > 0
            self.dataManager.setWindowLength(value)
        except : 
            pass
        
        # Rewrite the GUI with the current value
        self.updateWindowLengthGui() 
        
        
        
    def delayChanged(self):
        
        """ This function start the update of the delay in the thread manager
        when a changed has been detected """
        
        # Send the new value
        try : 
            value = float(self.delay_lineEdit.text())
            assert value >= 0
            self.monitorManager.setDelay(value)
        except : 
            pass
        
        # Rewrite the GUI with the current value
        self.updateDelayGui() 
        
        
        
    def updateWindowLengthGui(self):
        
        """ This function ask the current value of the window length in the data
        manager, and then update the GUI """
        
        value = self.dataManager.getWindowLength()
        self.windowLength_lineEdit.setText(f'{value:g}')
        self.setLineEditBackground(self.windowLength_lineEdit,'synced')
        
        
        
        
        
    def updateDelayGui(self):
        
        """ This function ask the current value of the delay in the data
        manager, and then update the GUI """
        
        value = self.monitorManager.getDelay()
        self.delay_lineEdit.setText(f'{value:g}')
        self.setLineEditBackground(self.delay_lineEdit,'synced')
        
        
        
    


        
        
        
        
        
        
        
    
    
    



        
        
        
        
        
        



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- thorlabs ITC4001
"""


class Driver():
    
    def __init__(self):
        pass

    def amplitude(self,amplitude):
        self.write(f'SOUR:CURR {amplitude}\n')
            
    def get_driver_model(self):
        config = []        
        config.append({'element':'variable','name':'amplitude','write':self.amplitude,'type':float,'help':"Set the pumping current value"})
        return config


#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='GPIB0::2::INSTR',**kwargs):
        import visa

        rm = visa.ResourceManager()
        self.inst = rm.get_instrument(address)
        Driver.__init__(self)
        
    def query(self, cmd):
        self.write(cmd)
        r = self.read()
        return r
    def read(self):
        return self.inst.read()
    def write(self,cmd):
        self.inst.write(cmd+'\n')
    def close(self):
        self.inst.close()
############################## Connections classes ##############################
#################################################################################
        



# -*- coding: utf-8 -*-
"""
Created on Sun Sep 29 18:13:45 2019

@author: qchat
"""

from . import main

class ParameterManager :
    
    def __init__(self,gui):
        
        self.gui = gui
        
        self.gui.parameterName_lineEdit.textEdited.connect(lambda : self.gui.setLineEditBackground(self.gui.parameterName_lineEdit,'edited'))
        self.gui.parameterName_lineEdit.returnPressed.connect(self.nameChanged)
        self.gui.parameterName_lineEdit.setEnabled(False)
        
        self.refresh()
        
        
        
    def refresh(self):
        
        """ This functions refreshes all the values of the name and address of the scan parameter, 
        from the configuration center """
                
        # Paramater name, address and unit
        parameter = self.gui.configManager.getParameter()
        
        if parameter is not None :
            name = self.gui.configManager.getParameterName()
            self.gui.parameterName_lineEdit.setEnabled(True)
            address = parameter.address()
            unit = parameter.unit
        else :
            name = ''
            self.gui.parameterName_lineEdit.setEnabled(False)
            address = ''
            unit = ''
            
        self.gui.parameterName_lineEdit.setText(name)
        self.gui.setLineEditBackground(self.gui.parameterName_lineEdit,'synced')
        self.gui.parameterAddress_label.setText(address)
        self.gui.startUnit_label.setText(unit)
        self.gui.meanUnit_label.setText(unit)
        self.gui.widthUnit_label.setText(unit)
        self.gui.endUnit_label.setText(unit)
        self.gui.stepUnit_label.setText(unit)
            
    
    
    def nameChanged(self):
        
        """ This function changes the name of the scan parameter """
        
        newName = self.gui.parameterName_lineEdit.text()
        newName = main.cleanString(newName)
        if newName != '' : 
            self.gui.configManager.setParameterName(newName)
            
            
            
    # PROCESSING STATE BACKGROUND
    ###########################################################################
    

    def setProcessingState(self,state):
        
        """ This function set the background color of the parameter address during the scan """
                
        if state == 'idle' : 
            self.gui.parameterAddress_label.setStyleSheet("font-size: 9pt;")
        else : 
            if state == 'started' : color = '#ff8c1a'
            if state == 'finished' : color = '#70db70'
            self.gui.parameterAddress_label.setStyleSheet(f"background-color: {color}; font-size: 9pt;")
            
        


# -*- coding: utf-8 -*-
"""
Created on Fri Oct 18 23:09:51 2019

@author: qchat
"""

def emphasize(txt,sign='-'):
    
    ''' Returns:    ---
                    txt
                    ---
    '''
    
    return sign*len(txt) + '\n' + txt + '\n' + sign*len(txt)


def underline(txt,sign='-'):
    
    ''' Returns:    
                    txt
                    ---
    '''
    
    return txt + '\n' + sign*len(txt)


def clean_string(txt):
    
    """ Returns txt without special characters """
    
    for character in '*."/\[]:;|, ' :
        txt = txt.replace(character,'')
        
    return txt   



def two_columns(txt_list) :
    
    ''' Returns a string of the form:
        txt[0]                         txt[1] 
        with a minimal spacing between the first character of txt1 and txt2 '''
    
    spacing = max([len(txt[0]) for txt in txt_list]) + 5
    
    return '\n'.join([ txt[0] + ' '*(spacing-len(txt[0])) + txt[1] 
            for txt in txt_list])




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""


import time 

class Driver():
        
    def __init__(self):
        pass
        self.chronometer_startTime = time.time()
    
    def chronometer(self):
        return time.time() - self.chronometer_startTime
    
    def reset_chronometer(self):
        self.chronometer_startTime = time.time()
        
    def parameter_buffer(self,value):
        pass
    
    def wait(self,delay):
        time.sleep(delay)
    
    def get_driver_model(self):
        
        model = []
        model.append({'element':'variable','name':'chronometer','type':float,'read':self.chronometer,'unit':'s','help':'Elapsed time since chronometer reset'})
        model.append({'element':'action','name':'reset_chronometer','do':self.reset_chronometer,'help':'Reset the chronometer time'})
        model.append({'element':'variable','name':'parameter_buffer','type':float,'write':self.parameter_buffer,'help':'Parameter buffer for monitoring in the scanning panel. Does nothing with the input value'})
        model.append({'element':'action','name':'wait','param_type':float,'param_unit':'s','do':self.wait,'help':'System wait during the value provided.'})
        return model
        
        
    
#################################################################################
############################## Connections classes ##############################
class Driver_DEFAULT(Driver):
    def __init__(self):
        
        Driver.__init__(self)
        

############################## Connections classes ##############################
#################################################################################



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""


class Driver():
    
    slot_config = '<MODULE_NAME>'

    def __init__(self,**kwargs):
        
        self.write('*RST') # The input buffer is cleared. The command interpreter is reset and a reset instruction is sent to every module. The status and event registers are cleared. Sets the OPC bit to 1.
        self.write('*CLS') # Clears the Event Status Register and the output queue. Sets the OPC bit to 1.
        
        # Submodules loading
        self.slot_names = {}
        prefix = 'slot'
        for key in kwargs.keys():
            if key.startswith(prefix) and not '_name' in key :
                slot_num = key[len(prefix):]
                module_name = kwargs[key].strip()
                module_class = globals()[f'Module_{module_name}']
                if f'{key}_name' in kwargs.keys() : name = kwargs[f'{key}_name']
                else : name = f'{key}_{module_name}'
                setattr(self,name,module_class(self,slot_num))
                self.slot_names[slot_num] = name
                print(self.slot_names)


    def get_id(self):
        return self.query('*IDN?')


    def get_driver_model(self):
        config = []
        for name in self.slot_names.values() :
            config.append({'element':'module','name':name,'object':getattr(self,name)})
        return config


#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='GPIB0::2::INSTR',**kwargs):
        
        import visa
        
        
        self.TIMEOUT = 60000 #ms
        
        rm = visa.ResourceManager()
        self.controller = rm.open_resource(address)
        self.controller.timeout = self.TIMEOUT
        
        Driver.__init__(self,**kwargs)
        

    def close(self):
        try : self.controller.close()
        except : pass


    def write(self,command):
        self.controller.write(command)
        
    def read(self):
        result = self.controller.read()
        result = result.strip('\n')
        return result
    
    def query(self,command):
        result = self.controller.query(command)
        result = result.strip('\n')
        return result
        
############################## Connections classes ##############################
#################################################################################


class Module_T100():   
    
    category = 'Optical source'
    
    def __init__(self,dev,slot):
        
        self.dev = dev
        self.SLOT = slot
        self.prefix = f'CH{int(slot)}:'
        
        self.write(self.prefix+'NM')
        self.write(self.prefix+'MW')


    def write(self,command):
        self.dev.write(command)
        
    def read(self):
        return self.dev.read()
    
    def query(self,command):
        return self.dev.query(command)
    
    #--------------------------------------------------------------------------
    # Optional functions
    #--------------------------------------------------------------------------
    
    def safe_state(self):
        self.set_output_state(False)
        if self.get_output_state() is False :
            return True
            

    def get_id(self):
        result = self.query(self.prefix+'*IDN?')
        result = self.clean_result(result)
        return result
        
        
    #--------------------------------------------------------------------------
    # Instrument variables
    #--------------------------------------------------------------------------
        
    def clean_result(self,result):
        try:
            result=result.split(':')[1]
            result=result.split('=')[1]
            result=float(result)
        except:
            pass
        return result
    
    


    def set_wavelength(self,value):
        self.write(self.prefix+f"L={value}")
        self.query('*OPC?')
        
    def get_wavelength(self):
        result = self.query(self.prefix+'L?')
        result = self.clean_result(result)
        return result
    
    
    
    
    
    def set_frequency(self,value):
        self.write(self.prefix+f"F={value}")
        self.query('*OPC?')
        
    def get_frequency(self):
        result = self.query(self.prefix+'F?')
        result = self.clean_result(result)
        return result
    
    
    
    
    
    
    def set_power(self,value):
        if value == 0 :
            self.set_output_state(False)
        else :
            if self.get_output_state() is False :
                self.set_output_state(True)
            self.write(self.prefix+f"P={value}")
            self.query('*OPC?')
            
    def get_power(self):
        result = self.query(self.prefix+'P?')
        result = self.clean_result(result)
        if result == 'Disabled':
            return 0
        else :
            return result
    




    
    def set_intensity(self,value):
        if value == 0 :
            self.set_output_state(False)
        else :
            if self.get_output_state() is False :
                self.set_output_state(True)
            self.write(self.prefix+f"I={value}")
            self.query('*OPC?')
        
    def get_intensity(self):
        result = self.query(self.prefix+'I?')
        result = self.clean_result(result)
        if result == 'Disabled':
            return 0
        else :
            return result
        
        
    
    
    
    def set_coherence_control_state(self,state):
        if state is True :
            self.write(self.prefix+'CTRL ON')
        else :
            self.write(self.prefix+'CTRL OFF')
        self.query('*OPC?')
        
    def get_coherence_control_state(self):
        result = self.query(self.prefix+'CTRL?')
        result = self.clean_result(result)
        return bool(int(result))
    
    
    
    
    def set_auto_peak_find_control_state(self,state):
        if state is True :
            self.write(self.prefix+'APF ON')
        else :
            self.write(self.prefix+'APF OFF')
        self.query('*OPC?')
        
    def get_auto_peak_find_control_state(self):
        result = self.query(self.prefix+'APF?')
        result = self.clean_result(result)
        return bool(int(result))
    
    
    
    
    
    def set_output_state(self,state):
        if state is True :
            self.write(self.prefix+"ENABLE")
        else :
            self.write(self.prefix+"DISABLE")
        self.query('*OPC?')
        
    def get_output_state(self):
        result = self.query(self.prefix+'ENABLE?')
        result = self.clean_result(result)
        return result == 'ENABLED'
    
    
    
    def get_driver_model(self):
        
        model = []
        model.append({'element':'variable','name':'wavelength','type':float,'unit':'nm','read':self.get_wavelength,'write':self.set_wavelength,'help':'Wavelength'})
        model.append({'element':'variable','name':'frequency','type':float,'unit':'GHz','read':self.get_frequency,'write':self.set_frequency,'help':'Frequency'})
        model.append({'element':'variable','name':'power','type':float,'unit':'mW','read':self.get_power,'write':self.set_power,'help':'Output power'})
        model.append({'element':'variable','name':'intensity','type':float,'unit':'mA','read':self.get_intensity,'write':self.set_intensity,'help':'Current intensity'})
        model.append({'element':'variable','name':'output','type':bool,'read':self.get_output_state,'write':self.set_output_state,'help':'Output state'})
        model.append({'element':'variable','name':'coherence_control','type':bool,'read':self.get_coherence_control_state,'write':self.set_coherence_control_state,'help':'Coherence control mode'})
        model.append({'element':'variable','name':'auto_peak_find_control','type':bool,'read':self.get_auto_peak_find_control_state,'write':self.set_auto_peak_find_control_state,'help':'Auto peak find control'})
        return model





class Module_SLD():   
    
    category = 'Broadband optical source'
    
    def __init__(self,dev,slot):
        
        self.dev = dev
        self.SLOT = slot
        self.prefix = f'CH{int(slot)}:'
        
        self.write(self.prefix+'NM')
        self.write(self.prefix+'MW')
        
    def write(self,command):
        self.dev.write(command)
        
    def read(self):
        return self.dev.read()
    
    def query(self,command):
        return self.dev.query(command)
    
    #--------------------------------------------------------------------------
    # Optional functions
    #--------------------------------------------------------------------------
        
    def safe_state(self):
        self.set_output_state(False)
        if self.get_output_state() is False :
            return True
            
        
    def get_id(self):
        return self.dev.query(self.prefix+'*IDN?')
        
        
    #--------------------------------------------------------------------------
    # Instrument variables
    #--------------------------------------------------------------------------

        
    def clean_result(self,result):
        try:
            result=result.split(':')[1]
            result=result.split('=')[1]
            result=float(result)
        except:
            pass
        return result
    
    

        
    def get_wavelength(self):
        result = self.query(self.prefix+'L?')
        result = self.clean_result(result)
        return result
    
    
   
    
    
    def set_power(self,value):
        if value < 5:
            self.set_output_state(False)
        elif 5<=value<10 :
            if self.get_output_state() is False :
                self.set_output_state(True)
            self.write(self.prefix+'P=LOW')
            self.query('*OPC?')
        else :
            if self.get_output_state() is False :
                self.set_output_state(True)
            self.write(self.prefix+'P=HIGH')
            self.query('*OPC?')   
            
            
        
    def get_power(self):
        result = self.query(self.prefix+'P?')
        result = self.clean_result(result)
        if result == 'Disabled':
            return 0
        elif result == 'HIGH':
            return 10
        elif result == 'LOW' :
            return 5
    
    
  
    
    
    def set_output_state(self,state):
        if state is True :
            self.write(self.prefix+"ENABLE")
        else :
            self.write(self.prefix+"DISABLE")
        self.query('*OPC?')
        
    def get_output_state(self):
        result = self.query(self.prefix+'ENABLE?')
        result = self.clean_result(result)
        return result == 'ENABLED'
    



    def get_driver_model(self):
        
        model = []
        model.append({'element':'variable','name':'power','type':float,'unit':'mW','read':self.get_power,'write':self.set_power,'help':'Output power'})
        model.append({'element':'variable','name':'output','type':bool,'read':self.get_output_state,'write':self.set_output_state,'help':'Output state'})
        return model
    
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""



class Driver():
    
    def __init__(self):
        self.write('*CLS')
        
        
    def get_current(self):
        return float(self.query(f'MEASure:CURRent?'))
    def get_current_compliance(self):
        return float(self.query(f'CURRent?'))
    def set_current_compliance(self,value):
        assert isinstance(float(value),float)
        value = float(value)
        self.write(f'CURRent {value}')
        self.query('*OPC?')

    def get_voltage(self):
        return float(self.query(f'MEASure:VOLTage?'))
    def get_voltage_compliance(self):
        return float(self.query(f'VOLTage?'))    
    def set_voltage_compliance(self,value):
        assert isinstance(float(value),float)
        value = float(value)
        self.write(f'VOLTage {value}')
        self.query('*OPC?')

    def get_output_state(self):
        ans = self.query(f"OUTPut?")
        return bool(int(float(ans)))
    def set_output_state(self,state):
        assert isinstance(state,bool)
        if state is True:
            self.write(f"OUTPut ON")
        else:
            self.write(f"OUTPut OFF") 
        self.query('*OPC?')
    
    def get_id(self):
        return self.query('*IDN?')
    
    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'current','unit':'A','read':self.get_current,'type':float,'help':'Current at the output as measured (read only)'})
        model.append({'element':'variable','name':'current_compliance','unit':'A','read':self.get_current_compliance,'write':self.set_current_compliance,'type':float,'help':'Current compliance'})
        model.append({'element':'variable','name':'voltage','unit':'V','read':self.get_voltage,'type':float,'help':'Voltage at the output as measured (read only)'})
        model.append({'element':'variable','name':'voltage_compliance','unit':'V','read':self.get_voltage_compliance,'write':self.set_voltage_compliance,'type':float,'help':'Voltage compliance'})
        model.append({'element':'variable','name':'output','read':self.get_output_state,'write':self.set_output_state,'type':bool,'help':'Output state (on/off), passed as boolean to the function: True/False'})
        return model
        
        
#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='GPIB0::22::INSTR',**kwargs):
        import visa

        self.TIMEOUT = 5000 #ms
        # Instantiation
        rm = visa.ResourceManager()
        self.controller = rm.get_instrument(address)
        self.controller.timeout = self.TIMEOUT
        
        Driver.__init__(self)
    
    
    def close(self):
        try : self.controller.close()
        except : pass
    def query(self,command):
        result = self.controller.query(command)
        result = result.strip('\n')
        return result
    def write(self,command):
        self.controller.write(command)
    def read(self):
        result = self.controller.read()
        return result.strip('\n')
    

class Driver_GPIB(Driver):
    def __init__(self,address=2,board_index=0,**kwargs):
        import Gpib
        
        self.inst = Gpib.Gpib(int(board_index),int(address))
        Driver.__init__(self, **kwargs)
    
    def query(self,query):
        self.write(query)
        return self.read()
    def write(self,command):
        self.inst.write(command)
    def read(self,length=1000000000):
        return self.inst.read(length).decode().strip('\r\n')
    def close(self):
        """WARNING: GPIB closing is automatic at sys.exit() doing it twice results in a gpib error"""
        #Gpib.gpib.close(self.inst.id)
        pass
############################## Connections classes ##############################
#################################################################################

        



# -*- coding: utf-8 -*-
"""
Created on Sun Sep 29 18:22:43 2019

@author: qchat
"""

import pandas as pd
import os

class DataManager :
    
    def __init__(self,gui):
        
        self.gui = gui
        self.windowLength = 10 #
        
        self.xlist = []
        self.ylist = []
        
        
        
        
    def setWindowLength(self,value):
        
        """ This function set the value of the window length """
        
        self.windowLength = value
        
        
        
    def getWindowLength(self):
        
        """ This function returns the value of the window legnth """
        
        return self.windowLength
    
    
    
    def getData(self):
        
        """ This function update the data of the provided plot object """
        
        return self.xlist,self.ylist
        
        
        
    def save(self,path):
        
        """ This function save the data in a file, in the provided path """
        
        df = pd.DataFrame({'Time [s]':self.xlist,f'{self.gui.variable.address()}':self.ylist})
        df.to_csv(os.path.join(path,'data.txt'),index=False)
        
        
        
    def addPoint(self,point):
        
        """ This function append a datapoint [x,y] in the lists of data """ 
        
        x,y = point
        
        # Append data
        self.xlist.append(x)
        self.ylist.append(y)
        
        # Remove too old data (regarding the window length)
        while max(self.xlist)-min(self.xlist) > self.windowLength : 
            self.xlist.pop(0)
            self.ylist.pop(0)
            

        
    


# -*- coding: utf-8 -*-
"""
Created on Fri Oct 18 22:20:14 2019

@author: qchat
"""

import os

import inspect
from .utilities import emphasize, clean_string
import autolab

class Element() :

    def __init__(self,parent,element_type,name):   

        self.name = name
        self._element_type = element_type
        self._parent = parent
        self._help = None
        
    def address(self):
    
        """ Returns the address of the given element.
        <module.submodule.variable> """
        
        if self._parent is not None : return self._parent.address()+'.'+self.name
        else : return self.name
    





class Module(Element):

    def __init__(self,parent,config):
        
        Element.__init__(self,parent,'module',config['name'])
        
        self._mod = {}
        self._var = {}
        self._act = {}
    
        # Object - instance
        assert 'object' in config.keys(), f"Module {self.name}: missing module object"
        self.instance = config['object']
        
        # Help
        if 'help' in config.keys():
            assert isinstance(config['help'],str), f"Module {self.address()} configuration: Help parameter must be a string"
            self._help = config['help']
     
        # Loading instance
        assert hasattr(self.instance,'get_driver_model'), f"There is no function 'get_driver_model' in the driver class"
        driver_config = self.instance.get_driver_model()

        assert isinstance(driver_config,list), f"Module {self.name} configuration: 'get_driver_model' output must be a list of dictionnaries"
        for config_line in driver_config : 
            
            # General check
            assert isinstance(config_line,dict), f"Module {self.name} configuration: 'get_driver_model' output must be a list of dictionnaries"
            
            # Name check
            assert 'name' in config_line.keys(), f"Module {self.name} configuration: missing 'name' key in one dictionnary"
            assert isinstance(config_line['name'],str), f"Module {self.name} configuration: elements names must be a string"
            name = clean_string(config_line['name'])
            assert name != '', f"Module {self.name}: elements names cannot be empty"
        
            # Element type check
            assert 'element' in config_line.keys(), f"Module {self.name}, Element {name} configuration: missing 'element' key in the dictionnary"
            assert isinstance(config_line['element'],str), f"Module {self.name}, Element {name} configuration: element type must be a string"
            element_type = config_line['element']
            assert element_type in ['module','variable','action'], f"Module {self.name}, Element {name} configuration: Element type has to be either 'module','variable' or 'action'"
            
            if element_type == 'module' :
                
                # Check name uniqueness
                assert name not in self.get_names(), f"Module {self.name}, Submodule {name} configuration: '{name}' already exists"
                self._mod[name] = Module(self,config_line)
                
            elif element_type == 'variable':
                
                # Check name uniqueness
                assert name not in self.get_names(), f"Module {self.name}, Variable {name} configuration: '{name}' already exists"
                self._var[name] = Variable(self,config_line)
                
            elif element_type == 'action' :
                
                # Check name uniqueness
                assert name not in self.get_names(), f"Module {self.name}, Action {name} configuration: '{name}' already exists"
                self._act[name] = Action(self,config_line)



    def get_module(self,name):
        
        """ Returns the submodule of the given name """
        
        assert name in self._mod.keys(), f"The submodule '{name}' does not exist in module {self.name}"
        return self._mod[name]
    
    
    
    def list_modules(self):
        
        """ Returns a list with the names of all existing submodules """
        
        return list(self._mod.keys())
    
    
    
    def get_variable(self,name):
        
        """ Returns the variable with the given name """
        
        assert name in self._var.keys(), f"The variable '{name}' does not exist in module {self.name}"
        return self._var[name]
    
    
    
    def list_variables(self):
        
        """ Returns a list with the names of all existing variables attached to this module """
        
        return list(self._var.keys())
    
    
    
    def get_action(self,name):
        
        """ Returns the action with the given name """
        
        assert name in self._dev.keys(), f"The action '{name}' does not exist in device {self.name}"
        return self._dev[name]
    
    
    
    def list_actions(self):
        
        """ Returns a list with the names of all existing actions attached to this module """
        
        return list(self._act.keys())
    
    
    
    def get_names(self):
        
        """ Returns the list of the names of all the elements of this module """
        
        return list(self._mod.keys()) + list(self._var.keys()) + list(self._act.keys())
    
    

    def __getattr__(self,attr):
        
        if attr in self._var.keys() : return self._var[attr]
        elif attr in self._act.keys() : return self._act[attr]
        elif attr in self._mod.keys() : return self._mod[attr]
        else : raise AttributeError(f"'{attr}' not found in module '{self.name}'")
        
        
        
    def sub_hierarchy(self,level=0):
        
        ''' Returns a list of the sub hierarchy of this module '''
        
        h = []
        
        from .devices import Device
        if isinstance(self,Device) : h.append([self.name,'Device/Module',level])
        else : h.append([self.name,'Module',level])
        
        for mod in self.list_modules() :
            h += self.get_module(mod).sub_hierarchy(level+1)
        for var in self.list_variables() :
            h.append([var,'Variable',level+1])
        for act in self.list_actions() :
            h.append([act,'Action',level+1])
            
        return h
        
        
    
    def help(self):
        
        """ This function prints informations for the user about the availables 
        submodules, variables and action attached to the current module """

        print(self)
        
    def __str__(self):
        
        """ This function returns informations for the user about the availables 
        submodules, variables and action attached to the current module """
        
        display ='\n'+emphasize(f'Module {self.name}')+'\n'
        if self._help is not None : 
            display+=f'Help: {self._help}\n'
            
        display += '\nElement hierarchy:'
            
        hierarchy = self.sub_hierarchy()
        for i, h_step in enumerate(hierarchy):
            if i == 0 : prefix = ''
            else : prefix = '|- '
            txt = ' '*3*h_step[2] + f'{prefix}{h_step[0]}'
            display += '\n' + txt + ' '*(30-len(txt)) + f'({h_step[1]})'
        
        return display
        
    def __dir__(self):
        
        """ For auto-completion """
        
        return self.list_modules() + self.list_variables() + self.list_actions() + ['help','instance']
    
    
    



        
        
class Variable(Element):
    
    def __init__(self,parent,config):
        
        Element.__init__(self,parent,'variable',config['name'])
        
        import numpy as np
        import pandas as pd
        
        # Type
        assert 'type' in config.keys(), f"Variable {self.address()}: Missing variable type"
        assert config['type'] in [int,float,bool,str,bytes,pd.DataFrame,np.ndarray], f"Variable {self.address()} configuration: Variable type not supported in autolab"
        self.type = config['type']
        
        # Read and write function
        assert 'read' in config.keys() or 'write' in config.keys(), f"Variable {self.address()} configuration: no 'read' nor 'write' functions provided"
        
        # Read function
        self.read_function = None
        if 'read' in config.keys():
            assert inspect.ismethod(config['read']), f"Variable {self.address()} configuration: Read parameter must be a function"
            self.read_function = config['read']
            
        # Write function
        self.write_function = None
        if 'write' in config.keys():
            assert inspect.ismethod(config['write']), f"Variable {self.address()} configuration: Write parameter must be a function"
            self.write_function = config['write']
        
        # Unit
        self.unit = None
        if 'unit' in config.keys():
            assert isinstance(config['unit'],str), f"Variable {self.address()} configuration: Unit parameter must be a string"
            self.unit = config['unit']
            
        # Help
        if 'help' in config.keys():
            assert isinstance(config['help'],str), f"Variable {self.address()} configuration: Info parameter must be a string"
            self._help = config['help']
                
        # Properties
        self.writable = self.write_function is not None
        self.readable = self.read_function is not None
        self.numerical = self.type in [int,float]
        self.parameter_allowed = self.writable and self.numerical
                
        # Signals for GUI
        self._read_signal = None
        self._write_signal = None
        
        
        
    def save(self,path,value=None):
        
        """ This function measure the variable and saves its value in the provided path """
        
        import pandas as pd
        import numpy as np
        
        assert self.readable, f"The variable {self.name} is not configured to be measurable"
        
        if os.path.isdir(path) :
            path = os.path.join(path,self.address()+'.txt')
        
        if value is None : value = self() # New measure if value not provided
        
        if self.type in [int,float,bool,str]:
            with open(path,'w') as f : f.write(str(value))
        elif self.type == bytes :
            with open(path,'wb') as f : f.write(value)
        elif self.type == np.ndarray :
            np.savetxt(path,value)
        elif self.type == pd.DataFrame :
            value.to_csv(path,index=False)
        else :
            raise ValueError("The variable {self.name} of type {self.type} cannot be saved.")  
        
        
        
    def help(self):
        
        """ This function prints informations for the user about the current variable """
    
        print(self)
    
    
    def __str__(self):
        
        """ This function returns informations for the user about the current variable """
        
        display ='\n'+emphasize(f'Variable {self.name}')+'\n'
        if self._help is not None : display+=f'Help: {self._help}\n'
        display += '\n'
        
        display += 'Readable: '
        if self.readable is True : display += f"YES (driver function '{self.read_function.__name__}')\n"
        else : display += 'NO\n'
        
        display += 'Writable: '
        if self.writable is True : display += f"YES (driver function '{self.write_function.__name__}')\n"
        else : display += 'NO\n'
        
        display+=f'Type: {self.type.__name__}\n'
        
        display += 'Unit: '
        if self.unit is not None : display += f'{self.unit}\n'
        else : display += 'None\n'
        
        return display
    
    

    def __call__(self,value=None):
        
        """ Measure or set the value of the variable """
                
        # GET FUNCTION
        if value is None:
            assert self.readable, f"The variable {self.name} is not readable"
            answer = self.read_function()
            if self._read_signal is not None : self._read_signal.emit(answer)
            return answer
            
        # SET FUNCTION
        else : 
            assert self.writable, f"The variable {self.name} is not writable"
            value = self.type(value)
            self.write_function(value)
            if self._write_signal is not None : self._write_signal.emit()
  
        
        

   
    
    
    
        
class Action(Element):
    
    def __init__(self,parent,config):
        
        Element.__init__(self,parent,'action',config['name'])
        
        import pandas as pd
        import numpy as np
        
        # Do function
        assert 'do' in config.keys(), f"Action {self.address()}: Missing 'do' function"
        assert inspect.ismethod(config['do']), f"Action {self.address()} configuration: Do parameter must be a function"
        self.function = config['do'] 
        
        # Argument
        self.type = None
        self.unit = None
        if 'param_type' in config.keys():
            assert config['param_type'] in [int,float,bool,str,bytes,pd.DataFrame,np.ndarray], f"Action {self.address()} configuration: Argument type not supported in autolab"
            self.type = config['param_type']
            if 'param_unit' in config.keys():
                assert isinstance(config['param_unit'],str), f"Action {self.address()} configuration: Argument unit parameter must be a string"
                self.unit = config['param_unit']
        
        # Help
        if 'help' in config.keys():
            assert isinstance(config['help'],str), f"Action {self.address()} configuration: Info parameter must be a string"
            self._help = config['help']
        
        self.has_parameter = self.type is not None
        
        
        
    def help(self):
        
        """ This function prints informations for the user about the current variable """
        
        print(self)
               
        
        
    def __str__(self):
        
        """ This function returns informations for the user about the current variable """
        
        display ='\n'+emphasize(f'Action {self.name}')+'\n'
        if self._help is not None : display+=f'Help: {self._help}\n'
        display += '\n'
        
        display+=f"Driver function: '{self.function.__name__}'\n"
        
        if self.has_parameter : 
            display += f'Parameter: YES (type: {self.type.__name__}) '
            if self.unit is not None : display += f'(unit: {self.unit})'
            display += '\n'
        else :
            display += 'Parameter: NO\n'
            
        return display
    
    
    
    def __call__(self,value=None):
        
        """ Executes the action """
        
        # DO FUNCTION
        assert self.function is not None, f"The action {self.name} is not configured to be actionable"
        if self.has_parameter :
            assert value is not None, f"The action {self.name} requires an argument"
            value = self.type(value)
            self.function(value)
        else :
            assert value is None, f"The action {self.name} doesn't require an argument"
            self.function()
            
            
    
        
        
        
        
        
      
        
        




# -*- coding: utf-8 -*-
"""
Created on Fri Sep 20 22:08:29 2019

@author: qchat
"""
from PyQt5 import QtWidgets, uic
import os

from .config import ConfigManager
from .figure import FigureManager
from .parameter import ParameterManager
from .recipe import RecipeManager
from .scan import ScanManager
from .data import DataManager
from .scanrange import RangeManager

class Scanner(QtWidgets.QMainWindow):
        
    def __init__(self,mainGui):
        
        self.mainGui = mainGui
        
        # Configuration of the window
        QtWidgets.QMainWindow.__init__(self)
        ui_path = os.path.join(os.path.dirname(__file__),'interface.ui')
        uic.loadUi(ui_path,self)
        self.setWindowTitle(f"AUTOLAB Scanner")
        
        # Loading of the different centers
        self.configManager = ConfigManager(self)
        self.figureManager = FigureManager(self)
        self.parameterManager = ParameterManager(self)
        self.recipeManager = RecipeManager(self)        
        self.scanManager = ScanManager(self)
        self.dataManager = DataManager(self)
        self.rangeManager = RangeManager(self)



    def closeEvent(self,event):
        
        """ This function does some steps before the window is really killed """
        
        # Stop ongoing scan
        if self.scanManager.isStarted() :
            self.scanManager.stop()
            
        # Stop datamanager timer
        self.dataManager.timer.stop()
        
        # Delete reference of this window in the control center
        self.mainGui.clearScanner()
        
        
        
    def setLineEditBackground(self,obj,state):
        
        """ Function used to set the background color of a QLineEdit widget, 
        based on its editing state """
        
        if state == 'synced' :
            color='#D2FFD2' # vert
        if state == 'edited' :
            color='#FFE5AE' # orange
            
        obj.setStyleSheet("QLineEdit:enabled {background-color: %s; font-size: 9pt}"%color)
        
        
        
        
        
def cleanString(name):
    
    """ This function clears the given name from special characters """
    
    for character in '*."/\[]:;|, ' :
        name = name.replace(character,'')
    return name        
    
    
        
        



        
    
        

        















     



        

        


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""

from numpy import zeros,ones,linspace


class Driver():
    
    def __init__(self):
        pass
    def amplitude(self,amplitude):
        self.write(f'VOLT {amplitude}')
    def get_amplitude(self):
        return float(self.query(f'VOLT?'))
    def offset(self,offset):
        self.write(f'VOLT:OFFS {offset}')
    def get_offset(self):
        return float(self.query(f'VOLT:OFFS?'))
    def frequency(self,frequency):
        self.write(f'FREQ {frequency}')
    def get_frequency(self):
        return float(self.query(f'FREQ?'))
    def ramp(self,ramp):
        l   = list(zeros(5000) - 1)
        lll = list(ones(5000))
        ll  = list(linspace(-1,1,100+ramp))
        l.extend(ll);l.extend(lll)
        s = str(l)[1:-1]
        self.write(f'DATA VOLATILE,{s}')

    def idn(self):
        self.write('*IDN?')
        return self.read()
        
    def get_driver_model(self):
        
        model = []
        model.append({'element':'variable','name':'amplitude','write':self.amplitude,'read':self.get_amplitude,'unit':'V','type':float,'help':'Amplitude'})
        model.append({'element':'variable','name':'offset','write':self.offset,'read':self.get_offset,'unit':'V','type':float,'help':'Offset'})
        model.append({'element':'variable','name':'frequency','write':self.frequency,'read':self.get_frequency,'unit':'Hz','type':float,'help':'Frequency'})
        
        return model


#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='GPIB0::2::INSTR',**kwargs):
        import visa
        
        rm = visa.ResourceManager()
        self.inst = rm.get_instrument(address)
        
        Driver.__init__(self)
        
    def close(self):
        self.inst.close()
    def query(self,query):
        self.write(query)
        return self.read()
    def write(self,command):
        self.inst.write(command)
    def read(self):
        rep = self.inst.read()
        return rep
        
class Driver_GPIB(Driver):
    def __init__(self,address=2,board_index=0,**kwargs):
        import Gpib
        
        self.inst = Gpib.Gpib(int(board_index),int(address))
        Driver.__init__(self, **kwargs)
    
    def query(self,query):
        self.write(query)
        return self.read()
    def write(self,command):
        self.inst.write(command)
    def read(self,length=1000000000):
        return self.inst.read(length).decode().strip('\r\n')
    def close(self):
        """WARNING: GPIB closing is automatic at sys.exit() doing it twice results in a gpib error"""
        #Gpib.gpib.close(self.inst.id)
        pass
############################## Connections classes ##############################
#################################################################################





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""


class Driver():
    
    def __init__(self):        
        pass
    
    def amplitude(self,amplitude):
        self.write('AMPL '+amplitude)
    def offset(self,offset):
        self.write('OFFS '+offset)
    def frequency(self,frequency):
        self.write('FREQ '+frequency)
    def phase(self,phase):
        self.write('PHSE '+phase)

    def idn(self):
        return self.query('*IDN?')
        
    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'amplitude','type':float,'unit':'V','write':self.amplitude,'help':'Voltage amplitude.'})
        model.append({'element':'variable','name':'offset','type':float,'unit':'V','write':self.offset,'help':'Voltage offset'})
        model.append({'element':'variable','name':'phase','type':float,'write':self.phase,'help':'Phase'})
        model.append({'element':'variable','name':'frequency','type':float,'unit':'Hz','write':self.frequency,'help':'Output frequency'})
        return model


#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='GPIB0::19::INSTR',**kwargs):
        import visa
        
        rm = visa.ResourceManager()
        self.inst = rm.get_instrument(address)
        
        Driver.__init__(self)
        
    def close(self):
        self.inst.close()
    def query(self,query):
        self.write(query)
        return self.read()
    def write(self,query):
        self.inst.write(query)
    def read(self):
        rep = self.inst.read()
        return rep
    
    
class Driver_GPIB(Driver):
    def __init__(self,address=19,board_index=0,**kwargs):
        import Gpib
        
        self.inst = Gpib.Gpib(int(board_index),int(address))
        Driver.__init__(self)
    
    def query(self,query):
        self.write(query)
        return self.read()
    def write(self,query):
        self.inst.write(query)
    def read(self,length=1000000000):
        return self.inst.read(length).decode().strip('\n')
    def close(self):
        """WARNING: GPIB closing is automatic at sys.exit() doing it twice results in a gpib error"""
        #Gpib.gpib.close(self.inst.id)
        pass
############################## Connections classes ##############################
#################################################################################





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""


class Driver():
    
    def __init__(self):
        pass
    
    def amplitude(self,amplitude):
        self.write(f'POW {amplitude}')
    def frequency(self,frequency):
        self.write(f'FREQ {frequency}')

    def idn(self):
        self.write('*IDN?')
        return self.read()
        
    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'amplitude','write':self.amplitude,'type':float,'help':"set the amplitude"})
        model.append({'element':'variable','name':'frequency','write':self.frequency,'type':float,'help':"Set the frequency"})
        return model

#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='TCPIP::192.168.0.3::INSTR', **kwargs):
        import visa as v
        
        rm        = v.ResourceManager()
        self.inst = rm.get_instrument(address)
        
        Driver.__init__(self)
        
    def query(self,query):
        self.write(query)
        return self.read()
    def write(self,query):
        self.inst.write(query)
    def read(self):
        return self.inst.read()
    def close(self):
        self.inst.close()
############################## Connections classes ##############################
#################################################################################





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""


class Driver():
    
    def __init__(self):
        pass

    def set_frequency(self, frequency):
        self.write(f'FR{frequency}HZ')

    def set_rfamp(self, amplitude):
        self.write(f'AP{amplitude}MV')

    def RFdisable(self):
        self.write('R2')

    def RFenable(self):
        self.write('R3')

    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'amplitude','write':self.set_rfamp,'type':float,'help':"set the amplitude"})
        model.append({'element':'variable','name':'frequency','write':self.set_frequency,'type':float,'help':"Set the frequency"})
        model.append({'element':'action','name':'RFenable','do':self.RFenable,'type':float,'help':"Enable RF output"})
        model.append({'element':'action','name':'RFdisable','do':self.RFdisable,'type':float,'help':"Disable RF output"})

        return model

#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='GPIB::7::INSTR', **kwargs):
        import visa as v
        
        r = v.ResourceManager()
        self.scope = r.get_instrument(address)
        Driver.__init__(self)
        
    def query(self, query, length=1000000):
        self.write(query)
        r = self.read(length=length)
        return r
    def write(self, query):
        self.string = query + '\n'
        self.scope.write(self.string)
    def read(self, length=10000000):
        rep = self.scope.read_raw()
        return rep
        
############################## Connections classes ##############################
#################################################################################




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""

import numpy as np
        
    

class Driver():
    
    def __init__(self):
        
        self.queue = []
    
        # Angle configuration : True = Shutter closed  ; False = Shutter open
        self.shutter1 = Shutter(self,1,{True:30,False:60})
        self.shutter2 = Shutter(self,2,{True:125,False:85})
        self.shutter3 = Shutter(self,3,{True:25,False:70})
                
        # Close every shutters at startup
        self.safe_state()       
        
        
    
    def get_id(self):
        
        ''' Returns the ID of the Arduino '''
        
        return self.query('ID?')
    
    
    
    def get_shutter(self,num):
        
        ''' Returns the shutter object with corresponding num '''
        
        return getattr(self,f'shutter{num}')
        
    
    
    def append_instruction(self,instruction):
        
        ''' Append a new element to the queue of instructions '''
        
        self.queue.append(instruction)
        
        
        
    def execute_queue(self):
        
        ''' Execute the queue of instructions '''
        
        if len(self.queue) > 0 :
            self.query(','.join(self.queue))
            self.queue = []
        
        
        
    def invert(self):
        
        ''' Invert the state of all shutters '''
        
        for num in range(3) : self.get_shutter(num+1).invert(execute=False)
        self.execute_queue()
      
        
        
    def safe_state(self):
        
        ''' Close every shutters '''
        
        for num in range(3) : self.get_shutter(num+1).set_state(True,execute=False)
        self.execute_queue()
        


    def set_global_state(self,global_state):
        
        ''' Set the given global_state '''
        
        assert isinstance(global_state,str)
        assert len(global_state) == 3
        for i in range(len(global_state)):
            if global_state[i] in ['0','1'] :
                num = i+1
                state = bool(int(global_state[i]))
                self.get_shutter(num).set_state(state,execute=False)
                
        self.execute_queue()
        


    def danse(self):
        
        ''' Little danse followed by a safe state '''
        
        for i in range(6) :
            for j in range(3) :
                self.get_shutter(j+1).set_angle(round(np.random.random()*180),execute=False)
        
        for j in range(3) :
            self.get_shutter(j+1).set_state(True, execute=False)
            self.get_shutter(j+1).set_state(True, execute=False)
                
        self.inst.timeout = 15000
        self.execute_queue()
        self.inst.timeout = 5000
        
        
        
        
    def get_driver_model(self):
        
        model = []
        
        for i in range(3):
            model.append({'element':'module','name':f'shutter{i+1}','object':getattr(self,f'shutter{i+1}')})
        
        model.append({'element':'variable','name':'global_state','type':str,'write':self.set_global_state,'help':'Global state of the shutters : "0x1" = Close shutter 1 and open shutter 3'})
        model.append({'element':'action','name':'safe_state','do':self.safe_state,'help':'Close every shutters'})
        model.append({'element':'action','name':'invert','do':self.invert,'help':'Invert every shutter state'})
        model.append({'element':'action','name':'danse','do':self.danse,'help':'Random danse then safe state'})
        return model
  
    
    
    
    
    
    
class Shutter :
    
    def __init__(self,master,num,angles):
        
        self.master = master
        self.num = num
        self.angles = angles
        self.state = True
        
        
        
    def set_angle(self,angle,execute=True):
        
        ''' Set the angle of the shutter '''
        
        self.master.append_instruction(f'SRV{self.num}={angle}')
        if execute is True : self.master.execute_queue()
        
        
        
    def set_state(self,state,**kwargs):
        
        ''' Set the state of the shutter '''
        
        self.set_angle(self.angles[state],**kwargs)
        self.state = state
        
        
                
    def get_state(self):
        
        ''' Return the current state of the shutter '''
        
        return self.state
    
    
    
    def invert(self,**kwargs):
        
        ''' Invert the current state of the shutter '''
        
        self.set_state(not(self.state),**kwargs)
        
        
        
    def get_driver_model(self):
        
        model = []
        model.append({'element':'variable','name':'state','type':bool,'read':self.get_state,'write':self.set_state,'help':'State of the shutter (True: beam blocked)'})
        model.append({'element':'action','name':'invert','do':self.invert,'help':'Invert the shutter state'})
        return model
        
    
    


#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='ASRL::2::INSTR', **kwargs):
        import visa 
        
        self.BAUDRATE = 115200 

        # Instantiation
        rm = visa.ResourceManager()
        self.inst = rm.open_resource(address)
        self.inst.timeout = 5000 #ms
        self.inst.baud_rate = self.BAUDRATE
        
        Driver.__init__(self)
        
        
    def close(self):
        try : self.inst.close()
        except : pass

    def query(self,command):
        return self.inst.query(command).strip('\n')
    
    def write(self,command):
        self.inst.write(command)

############################## Connections classes ##############################
#################################################################################
        

        



# -*- coding: utf-8 -*-
"""
Created on Sun Sep 29 18:22:35 2019

@author: qchat
"""

from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
import matplotlib
import os
            
class FigureManager :
    
    def __init__(self,gui):
        
        self.gui = gui
        
        # Configure and initialize the figure in the GUI
        self.fig = Figure()
        matplotlib.rcParams.update({'font.size': 12})
        self.ax = self.fig.add_subplot(111)        
        self.ax.set_xlabel('Time [s]')
        ylabel = f'{self.gui.variable.address()}'
        if self.gui.variable.unit is not None :
            ylabel += f' ({self.gui.variable.unit})'
        self.ax.set_ylabel(ylabel)
        self.ax.grid()
        self.plot=self.ax.plot([0],[0],'x-',color='r')[0]
        # The first time we open a monitor it doesn't work, I don't know why..
        # There is no current event loop in thread 'Thread-7'.
        # More accurately, FigureCanvas doesn't find the event loop the first time it is called
        # The second time it works..
        # Seems to be only in Spyder..
        try : 
            self.canvas = FigureCanvas(self.fig) 
        except :
            self.canvas = FigureCanvas(self.fig)
        self.gui.graph.addWidget(self.canvas)
        self.fig.tight_layout()
        self.canvas.draw()

    
    def update(self,xlist,ylist):
        
        """ This function update the figure in the GUI """ 
        
        # Data retrieval
        self.plot.set_xdata(xlist)
        self.plot.set_ydata(ylist)

        # X axis update
        xlist = self.plot.get_xdata()
        xmin = min(xlist)
        xmax = max(xlist)
        if xmin != xmax :
            self.ax.set_xlim(xmin,xmax+0.15*(xmax-xmin))
        else :
            self.ax.set_xlim(xmin-0.1,xmin+0.1)
        
        # Y axis update
        ylist = self.plot.get_ydata()
        ymin = min(ylist)
        ymax = max(ylist)
        if ymin != ymax :
            self.ax.set_ylim(ymin-0.1*(ymax-ymin),ymax+0.1*(ymax-ymin))
        else :
            self.ax.set_ylim(ymin-0.1,ymin+0.1)
            
        # Figure finalization  
        self.redraw()
        #self.canvas.draw()

            
            
        
    def redraw(self):
        
        """ This function finalize the figure update in the GUI """
        
        try :
            self.fig.tight_layout()
        except :
            pass
        self.canvas.draw()
        
        
        
        
                
    def save(self,path):
        
        """ This function save the figure in the provided path """
        
        self.fig.savefig(os.path.join(path,'figure.jpg'),dpi=300)
        
        
        
        


# -*- coding: utf-8 -*-
"""
Created on Fri May 17 16:36:53 2019

@author: quentin.chateiller
"""
import numpy as np
import time
import pandas as pd



class Driver():
    
    def __init__(self):
        
        self.amp = 1
        
        self.slot1 = Slot(self,1)
        self.slot2 = Slot(self,2)
        
        self.option = True
        
        self.count = 0
        self.phrase = 'Coucou'
        self.sleep = 0
        #raise ValueError('Test error')
        
        self.verbose = False
        
    def set_sleep(self,value):
        self.sleep = value
        
    def get_sleep(self):
        return self.sleep
    
    def set_verbose(self,value):
        self.verbose = value
        
    def get_verbose(self):
        return self.verbose
    
    def get_amplitude(self):
        time.sleep(self.sleep)
        #raise ValueError('Test error')
        #self.count += 1
#        if np.random.uniform() > 0.5 : sign = 1
#        else : sign = -1
#        self.amp = self.amp*(1+sign*np.random.uniform()*0.1)
#        if self.count == 100 :
#            raise ValueError()
        value = self.amp + np.random.uniform(-1, 1)*0.01
        
        if self.verbose : print('get amplitude',value)

        return value
    
    def set_phrase(self,phrase):
        time.sleep(self.sleep)
        assert isinstance(phrase,str)
        self.phrase = phrase
        if self.verbose : print('set phrase',self.phrase)
        
    def get_phrase(self):
        time.sleep(self.sleep)
        if self.verbose : print('get phrase',self.phrase)
        return self.phrase
    
    def set_amplitude(self,value):
        time.sleep(self.sleep)
        self.amp = value
        if self.verbose : print('set amplitude',self.amp)
        #raise ValueError('Test error')
    
    def close(self):
        if self.verbose : print('DUMMY DEVICE CLOSED')
        
    def get_phase(self):
        time.sleep(self.sleep)
        value = np.random.uniform(-1, 1)
        if self.verbose : print('get phase',value)
        return value
    
    def set_phase(self,value):
        time.sleep(self.sleep)
        self.phase = value
        if self.verbose : print('set phase',value)
    
    def do_sth(self):
        time.sleep(self.sleep)
        if self.verbose : print('do sth')
        #raise ValueError('Test error')
        
    def get_dataframe(self):
        df = pd.DataFrame()
        d = {'e':1,'f':2}
        df=df.append(d,ignore_index=True)
        time.sleep(self.sleep)
        if self.verbose : print('get dataframe',d)
        return df
    
    def set_option(self,value):
        time.sleep(self.sleep)
        self.option = bool(value)
        if self.verbose : print('set option',self.option)
        
    def get_option(self):
        time.sleep(self.sleep)
        if self.verbose : print('get option',self.option)
        return self.option
    
    def get_array(self):
        time.sleep(self.sleep)
        return np.ones((3,4))
    
    def get_driver_model(self):
        
        model = []
        
        for i in range(10) :
            if hasattr(self,f'slot{i}') :
                model.append({'element':'module','name':f'slot{i}','object':getattr(self,f'slot{i}')})
        
        model.append({'element':'variable','name':'amplitude','type':float,'unit':'V','read':self.get_amplitude,'write':self.set_amplitude,'help':'This is the amplitude of the device...'})
        model.append({'element':'variable','name':'phrase','type':str,'read':self.get_phrase,'write':self.set_phrase})
        model.append({'element':'variable','name':'phase','type':float,'read':self.get_phase,'write':self.set_phase})
        model.append({'element':'action','name':'something','do':self.do_sth,'help':'This do something...'})
        model.append({'element':'variable','name':'dataframe','type':pd.DataFrame,'read':self.get_dataframe})
        model.append({'element':'variable','name':'option','type':bool,'read':self.get_option,'write':self.set_option})
        model.append({'element':'variable','name':'array','type':np.ndarray,'read':self.get_array})
        model.append({'element':'variable','name':'sleep','type':float,'read':self.get_sleep,'write':self.set_sleep})
        model.append({'element':'variable','name':'verbose','type':bool,'read':self.get_verbose,'write':self.set_verbose})
        
        return model
    
    
class Driver_CONN(Driver):
    
    def __init__(self,address='192.168.0.8',**kwargs):
        print('DUMMY DEVICE INSTANTIATED with address',address)
        
        Driver.__init__(self)
        
    
class Slot() :
    
    def __init__(self,dev,num):
        self.dev = dev
        self.num = num
        
    def get_power(self):
        time.sleep(self.dev.sleep)
        value = np.random.uniform()
        if self.dev.verbose : print(f'slot {self.num} get power',value)
        return value
    
    def get_wavelength(self):
        time.sleep(self.dev.sleep)
        value = np.random.uniform()
        if self.dev.verbose : print(f'slot {self.num} get wavelength',value)
        return value

    def get_driver_model(self):
        config = []
        config.append({'element':'variable','name':'power','type':float,'read':self.get_power,'unit':'W'})
        config.append({'element':'variable','name':'wavelength','type':float,'read':self.get_wavelength,'unit':'nm'})
        return config



# -*- coding: utf-8 -*-
"""
Created on Fri Sep 20 17:11:57 2019

@author: qchat
"""

import os 

USER_FOLDER = os.path.join(os.path.expanduser('~'),'autolab')
USER_LAST_CUSTOM_FOLDER = os.path.expanduser('~')
LOCAL_CONFIG = os.path.join(USER_FOLDER,'local_config.ini')
AUTOLAB_CONFIG = os.path.join(USER_FOLDER,'autolab_config.ini')
LOCAL_CONFIG_TEMPLATE = os.path.join(os.path.dirname(__file__),'local_config.ini')
        
DRIVER_SOURCES = {'main':os.path.join(os.path.dirname(os.path.dirname(__file__)),'drivers'),
                  'local':os.path.join(USER_FOLDER,'local_drivers')}


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Electrical source'
    
    
class Driver_parser():
    def __init__(self, Instance, name, **kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}

----------------  Examples:  ----------------

usage:    autolab driver [options] args
            
    autolab driver -D {self.name} -A GPIB::7::INSTR -C VISA -v 0.2 -c A,B
    load {self.name} driver using VISA communication protocol with address TCPIP... and set the voltage to 0.2V to channel A and B
    
    autolab driver -D nickname -v 0.2 -c A
    same as before but using the device nickname as defined in local_config.ini
            """
        return usage
    
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        parser.add_argument("-c", "--channels", type=str, dest="channels", default=None, help="Set the channels to act on." )
        parser.add_argument("-v", "--voltage", type=str, dest="voltage", default=None, help="Set the voltage in V." )
        parser.add_argument("-a", "--current", type=str, dest="current", default=None, help="Set the current in A." )
        
        return parser

    def do_something(self,args):
        if args.channels:
            for chan in args.channels.split(','):
                if args.voltage:
                    getattr(getattr(self.Instance,f'channel{chan}'),'set_voltage')(args.voltage)
                if args.current:
                    getattr(getattr(self.Instance,f'channel{chan}'),'set_current')(args.current)
            

    def exit(self):
        self.Instance.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""



class Driver():
    
    def __init__(self):
        
        # Initialisation
        self.write('*CLS')
        
        # Subdevices
        self.channelA = Channel(self,'a')
        self.channelB = Channel(self,'b')
    
    def safe_state(self):
        self.channelA.safe_state()
        self.channelB.safe_state()
    
    def get_id(self):
        return self.query('*IDN?')
        
    def get_driver_model(self):
        model = []
        model.append({'element':'module','name':'channelA','object':self.channelA})
        model.append({'element':'module','name':'channelB','object':self.channelB})
        return model
 

#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='GPIB0::26::INSTR',**kwargs):
        import visa

        self.TIMEOUT = 5000 #ms
        
        # Instantiation
        rm = visa.ResourceManager()
        self.controller = rm.open_resource(address)
        self.controller.timeout = self.TIMEOUT
        
        Driver.__init__(self)
    
        
   
    def close(self):
        try : self.controller.close()
        except : pass

    def query(self,command):
        result = self.controller.query(command)
        result = result.strip('\n')
        return result
    
    def write(self,command):
        self.controller.write(command)
        
        
############################## Connections classes ##############################
#################################################################################



class Channel():
    
    def __init__(self,dev,slot):
        
        self.dev = dev
        self.SLOT = slot.lower()
        
        # Initialisation
        self.dev.write(f"smu{self.SLOT}.source.autorangev = smu{self.SLOT}.AUTORANGE_ON")
        self.dev.write(f"smu{self.SLOT}.source.autorangei = smu{self.SLOT}.AUTORANGE_ON")
        self.dev.query('*OPC?')
    
    def safe_state(self):
        self.set_voltage(0)
        
        
        
    #--------------------------------------------------------------------------
    # Instrument variables
    #--------------------------------------------------------------------------
        


    def get_resistance(self):
        self.dev.write(f'display.smu{self.SLOT}.measure.func = display.MEASURE_OHMS')
        self.dev.query('*OPC?')
        return float(self.dev.query(f"print(smu{self.SLOT}.measure.r())"))
    
    
    
    
    def get_power(self):
        self.dev.write(f'display.smu{self.SLOT}.measure.func = display.MEASURE_WATTS')
        self.dev.query('*OPC?')
        return float(self.dev.query(f"print(smu{self.SLOT}.measure.p())"))




    def set_power_compliance(self,value):
        assert isinstance(float(value),float)
        value = float(value)
        self.dev.write(f"smu{self.SLOT}.source.limitp = {value}")
        self.dev.query('*OPC?')
        
    def get_power_compliance(self):
        return float(self.dev.query(f"print(smu{self.SLOT}.source.limitp)"))




    
    def get_current(self):
        self.dev.write(f'display.smu{self.SLOT}.measure.func = display.MEASURE_DCAMPS')
        return float(self.dev.query(f"print(smu{self.SLOT}.measure.i())"))
    
    def set_current(self,value):
        assert isinstance(float(value),float)
        value = float(value)
        self.dev.write(f"smu{self.SLOT}.source.func = smu{self.SLOT}.OUTPUT_DCAMPS")
        self.dev.write(f"smu{self.SLOT}.source.leveli = {value}")
        self.dev.query('*OPC?')
#        if value != 0. and self.get_output_state() is False :
#            self.set_output_state(True)
#        if value == 0. and self.get_output_state() is True :
#            self.set_output_state(False)
            
            
            
            
            
    def set_current_compliance(self,value):
        assert isinstance(float(value),float)
        value = float(value)
        self.dev.write(f"smu{self.SLOT}.source.limiti = {value}")
        self.dev.query('*OPC?')
        
    def get_current_compliance(self):
        return float(self.dev.query(f"print(smu{self.SLOT}.source.limiti)"))





    
    def get_voltage(self):
        self.dev.write(f'display.smu{self.SLOT}.measure.func = display.MEASURE_DCVOLTS')
        self.dev.query('*OPC?')
        return float(self.dev.query(f"print(smu{self.SLOT}.measure.v())"))
    
    def set_voltage(self,value):
        assert isinstance(float(value),float)
        value = float(value)
        self.dev.write(f"smu{self.SLOT}.source.func = smu{self.SLOT}.OUTPUT_DCVOLTS")
        self.dev.write(f"smu{self.SLOT}.source.levelv = {value}")
        self.dev.query('*OPC?')
#        if value != 0. and self.get_output_state() is False :
#            self.set_output_state(True)
#        if value == 0. and self.get_output_state() is True :
#            self.set_output_state(False)
            
            
            
    def set_voltage_compliance(self,value):
        assert isinstance(float(value),float)
        value = float(value)
        self.dev.write(f"smu{self.SLOT}.source.limitv = {value}")
        self.dev.query('*OPC?')
        
    def get_voltage_compliance(self):
        return float(self.dev.query(f"print(smu{self.SLOT}.source.limitv)"))





        
    def get_output_state(self):
        ans = self.dev.query(f"print(smu{self.SLOT}.source.output)")
        return bool(int(float(ans)))
                
    def set_output_state(self,state):
        assert isinstance(state,bool)
        if state is True :
            self.dev.write(f"smu{self.SLOT}.source.output = smu{self.SLOT}.OUTPUT_ON")
        else :
            self.dev.write(f"smu{self.SLOT}.source.output = smu{self.SLOT}.OUTPUT_OFF") 
        self.dev.query('*OPC?')
        
    
            

    
    def set_4wire_mode_state(self,state):
        assert isinstance(state,bool)
        if state is True :
            self.dev.write(f'smu{self.SLOT}.sense = smu{self.SLOT}.SENSE_REMOTE')
        else :
            self.dev.write(f'smu{self.SLOT}.sense = smu{self.SLOT}.SENSE_LOCAL')  
        self.dev.query('*OPC?')

    def get_4wire_mode_state(self):
        result=int(float(self.dev.query(f"print(smu{self.SLOT}.sense)")))
        if result == 0 :
            return False
        else :
            return True
        
        
    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'resistance','unit':'ohm','read':self.get_resistance,'type':float,'help':'Resistance'})
        model.append({'element':'variable','name':'power','unit':'W','read':self.get_power,'type':float,'help':'Power'})
        model.append({'element':'variable','name':'power_compliance','unit':'W','read':self.get_power_compliance,'write':self.set_power_compliance,'type':float,'help':'Power compliance'})
        model.append({'element':'variable','name':'current','unit':'A','read':self.get_current,'write':self.set_current,'type':float,'help':'Current'})
        model.append({'element':'variable','name':'current_compliance','unit':'A','read':self.get_current_compliance,'write':self.set_current_compliance,'type':float,'help':'Current compliance'})
        model.append({'element':'variable','name':'voltage','unit':'V','read':self.get_voltage,'write':self.set_voltage,'type':float,'help':'Voltage'})
        model.append({'element':'variable','name':'voltage_compliance','unit':'V','read':self.get_voltage_compliance,'write':self.set_voltage_compliance,'type':float,'help':'Voltage compliance'})
        model.append({'element':'variable','name':'output','read':self.get_output_state,'write':self.set_output_state,'type':bool,'help':'Output'})
        model.append({'element':'variable','name':'4wire_mode','read':self.get_4wire_mode_state,'write':self.set_4wire_mode_state,'type':bool,'help':'4 wire mode'})
        return model
        



# -*- coding: utf-8 -*-
"""
Created on Sun Sep 29 18:10:31 2019

@author: qchat
"""

import collections
from queue import Queue
from PyQt5 import QtCore,QtWidgets
from autolab import paths
from distutils.dir_util import copy_tree
import os 
import pandas as pd
import tempfile

class DataManager :
    
    def __init__(self,gui):
        
        self.gui = gui
        
        self.datasets = []
        self.queue = Queue()
        self.initialized = False
        
        # Timer
        self.timer = QtCore.QTimer(self.gui)
        self.timer.setInterval(33) #30fps
        self.timer.timeout.connect(self.sync)
        
        # Save button configuration
        self.gui.save_pushButton.clicked.connect(self.saveButtonClicked)
        self.gui.save_pushButton.setEnabled(False)
        
        # Clear button configuration
        self.gui.clear_pushButton.clicked.connect(self.clear)
        
        
        
    def getData(self,nbDataset,varList):
        
        """ This function returns to the figure manager the required data """
        
        dataList = []
        
        for i in range(nbDataset) :
            if i < len(self.datasets) :
                dataset = self.datasets[-(i+1)]
                data = dataset.getData(varList)
                dataList.append(data)
            else :
                break
            
        dataList.reverse()
        
        return dataList
        
        
        
    def saveButtonClicked(self):
        
        """ This function is called when the save button is clicked. 
        It asks a path and starts the procedure to save the data """
        
        dataset = self.getLastDataset()
        if dataset is not None :
            
            path = str(QtWidgets.QFileDialog.getExistingDirectory(self.gui, 
                                                              "Select Directory",
                                                              paths.USER_LAST_CUSTOM_FOLDER))
     
            if path != '' :
                self.gui.statusBar.showMessage(f'Saving data...',5000)
                
                dataset.save(path)
                self.gui.figureManager.save(path)
                
                self.gui.statusBar.showMessage(f'Last dataset successfully saved in {path}',5000)
        
        

    def clear(self):
        
        """ This reset any recorded data, and the GUI accordingly """
        
        self.datasets = []
        self.initialized = False
        self.gui.figureManager.clearData()
        self.gui.variable_x_comboBox.clear()
        self.gui.variable_y_comboBox.clear()
        self.gui.save_pushButton.setEnabled(False)
        
        
        
    def getLastDataset(self):
        
        """ This return the current (last created) dataset """
        
        if len(self.datasets)>0 :
            return self.datasets[-1]
        
        
        
    def newDataset(self,config):
        
        """ This function creates and returns a new empty dataset """

        dataset = Dataset(self.gui,config)
        self.datasets.append(dataset)
        self.gui.progressBar.setMaximum(config['nbpts'])
        return dataset
    
    
    
    def sync(self):
        
        """ This function sync the last dataset with the data available in the queue """
        
        # Empty the queue
        count = 0
        dataset = self.getLastDataset()
        lenQueue = self.queue.qsize()
        for i in range(lenQueue) :
            try : point = self.queue.get()
            except : break
            dataset.addPoint(point)
            count += 1
        
        # Upload the plot if new data available
        if count > 0 :
            
            # Updagte progress bar
            self.gui.progressBar.setValue(len(dataset))
            
            # Executed after the first start of a new config scan
            if self.initialized is False :
                self.updateDisplayableResults()
                self.gui.save_pushButton.setEnabled(True)
                self.initialized = True
                
            # Executed after any dataset newly created and fed
            if dataset.new is True :
                self.gui.figureManager.reloadData()
                dataset.new = False
                
            # Executed each time the queue is read
            self.gui.figureManager.reloadLastData()

        

    def updateDisplayableResults(self) :
        
        """ This function update the combobox in the GUI that displays the names of
        the results that can be plotted """
        
        dataset = self.getLastDataset()
        
        resultNamesList = []
        for resultName in dataset.data.columns :
            if resultName not in ['id'] :
                try : 
                    float(dataset.data.iloc[0][resultName])
                    resultNamesList.append(resultName)
                except : 
                    pass
                
    
        self.gui.variable_x_comboBox.clear()
        self.gui.variable_x_comboBox.addItems(resultNamesList) # parameter first
        
        name=resultNamesList.pop(0)
        resultNamesList.append(name)
        self.gui.variable_y_comboBox.clear()
        self.gui.variable_y_comboBox.addItems(resultNamesList) # first numerical measure first
        
        
        
        
class Dataset():
    
    def __init__(self,gui,config):
        
        self.gui = gui
        self.config = config
     
        self.data = pd.DataFrame()
        self.tempFolderPath = tempfile.mkdtemp() # Creates a temporary directory for this dataset
        self.new = True
        
        # Save current config in the temp directory
        self.gui.configManager.export(os.path.join(self.tempFolderPath,'config.conf'))
        
        

    def getData(self,varList):
        
        """ This function returns a dataframe with two columns : the parameter value,
        and the requested result value """

        if varList[0] == varList[1] : return self.data.loc[:,[varList[0]]]
        else : return self.data.loc[:,varList]
    
    
    
    def save(self,path):
        
        """ This function saved the dataset in the provided path """
        
        copy_tree(self.tempFolderPath,path)
        
    
    
    def addPoint(self,dataPoint):
        
        """ This function add a data point (parameter value, and results) in the dataset """
        
        ID = len(self.data)+1
        
        simpledata =collections.OrderedDict()
        simpledata['id']=ID
        
        for resultName in dataPoint.keys():
            
            result = dataPoint[resultName]
            
            if resultName == self.config['parameter']['name'] :
                element = self.config['parameter']['element']
            else :
                element = [step['element'] for step in self.config['recipe'] if step['name']==resultName][0]
            
            resultType = element.type
            
            # If the result is displayable (numerical), keep it in memory
            if resultType in [int,float,bool]:
               simpledata[resultName] = result
                
            # Else write it on a file, in a temp directory
            else : 
                folderPath = os.path.join(self.tempFolderPath,resultName)
                if os.path.exists(folderPath) is False : os.mkdir(folderPath)
                filePath = os.path.join(folderPath,f'{ID}.txt')
                element.save(filePath,value=result)
                
            
        # Store data in the dataset's dataframe
        self.data = self.data.append(simpledata,ignore_index=True)
        if ID == 1 : 
            self.data = self.data[list(simpledata.keys())] # reorder columns
            header = True
        else :
            header = False
        self.data.tail(1).to_csv(os.path.join(self.tempFolderPath,f'data.txt'),index=False,mode='a',header=header)
        
        
        
    def __len__(self):
        
        """ Returns the number of data point of this dataset """
        
        return len(self.data)
    
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Electrical frame'

class Driver_parser():
    def __init__(self, Instance, name, **kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}

----------------  Examples:  ----------------

usage:    autolab driver [options] args
            
    autolab driver -D {self.name} -C VISA -A GPIB0::2::INSTR -c 3,5 -s 2
    load {self.name} driver using VISA and local network address 192.168.0.4 and set setpoint of SIM960(PID controller) to 2V module inserted in slots 3 and 5
    
    autolab driver -D nickname -c 2 -r 
    Similar to previous one but using the device's nickname as defined in local_config.ini and set output to pid control.
    
    autolab driver -D nickname -c 2 -u
    Similar to previous one but using the device's nickname as defined in local_config.ini and set output to manual.
    
    autolab driver -D nickname -c 2 -a
    Similar to previous one but using the device's nickname as defined in local_config.ini and set output to manual to unlock and try relock.
    
    Note: Arbitrary waveform available only using a python terminal
            """
        return usage
    
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        parser.add_argument("-c", "--channels",type=str, dest="channels", default=None, help="Set the slots to act on/acquire from." )
        parser.add_argument("-s", "--setpoint",type=str, dest="setpoint", default=None, help="Setpoint value to be used in Volts (for SIM960 PID controller)" )
        parser.add_argument("-r", "--relock",action="store_true", dest="lock", default=False, help="Lock (for SIM960 PID controller)" )
        parser.add_argument("-u", "--unlock",action="store_true", dest="unlock", default=False, help="Unlock (for SIM960 PID controller)" )
        parser.add_argument("-a", "--auto_lock",action="store_true", dest="auto_lock", default=False, help="Choose automatically to unlock and relock in order to decrease the output voltage (for SIM960 PID controller)" )
        
        return parser

    def do_something(self,args):
        if args.channels:
            for chan in args.channels.split(','):
                assert f'{chan}' in getattr(self.Instance,f'slot_names').keys()
                name_sub_module = getattr(self.Instance,f'slot_names')[f'{chan}']
                sub_module = getattr(self.Instance,name_sub_module)
                if args.setpoint:
                    func_name = 'set_setpoint'
                    assert hasattr(sub_module,func_name), "Module has no attribute {func_name}, are you addressing the right slot?"
                    getattr(sub_module,func_name)(args.setpoint)
                if args.relock:
                    func_name = 'relock'
                    assert hasattr(sub_module,func_name), "Module has no attribute {func_name}, are you addressing the right slot?"
                    getattr(sub_module,func_name)()
                elif args.unlock:
                    func_name = 'set_output_manual'
                    assert hasattr(sub_module,func_name), "Module has no attribute {func_name}, are you addressing the right slot?"
                    getattr(sub_module,func_name)()
                elif args.auto_lock:
                    func_name = 'auto_lock'
                    assert hasattr(sub_module,func_name), "Module has no attribute {func_name}, are you addressing the right slot?"
                    getattr(sub_module,func_name)()


    def exit(self):
        self.Instance.close()




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""


class Driver():
    
    def __init__(self,nb_channels=2):
        
        self.nb_channels = int(nb_channels)
        
        for i in range(1,self.nb_channels+1):
            setattr(self,f'channel{i}',Channel(self,i))
        
    
    def idn(self):
        return self.query('*IDN?')
        
    def get_driver_model(self):
        model = []
        for i in range(1,self.nb_channels+1):
            model.append({'element':'module','name':f'channel{i}','object':getattr(self,f'channel{i}'), 'help':'Channels to address command to'})
        return model


#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='TCPIP::192.168.0.3::INSTR', **kwargs):
        import visa as v
        
        rm        = v.ResourceManager()
        self.inst = rm.get_instrument(address)
        Driver.__init__(self, **kwargs)
        
    def query(self,query):
        self.write(query)
        return self.read()
    def write(self,query):
        self.inst.write(query)
    def read(self):
        rep = self.inst.read()
        return rep
    def close(self):
        self.write(':SYST:COMM:RLST LOC')
        self.inst.close()
############################## Connections classes ##############################
#################################################################################


class Channel():
    def __init__(self,dev,channel):
        self.channel = int(channel)
        self.dev     = dev


    def arbitrary_mode(self,waveform,round_factor=4):
        assert len(waveform) <= 524288, "Don't overcome Sample max of 524288"
        waveform = ''.join([str(round(waveform[i],round_factor))+',' for i in range(len(waveform))])[:-1]
        self.dev.write(f'DATA{self.channel} VOLATILE,{waveform}')
    def amplitude(self,amplitude):
        self.dev.write(f':VOLT{self.channel} {amplitude}')
    def offset(self,offset):
        self.dev.write(f':VOLT{self.channel}:OFFS {offset}')
    def frequency(self,frequency):
        self.dev.write(f':FREQ{self.channel} {frequency}')
    
    def set_mode(self,mode_name):
        self.dev.write(f':FUNC{self.channel} {mode_name}')
    def get_mode(self):
        return self.dev.query(f':FUNC{self.channel}?')
    
    def set_pulse_duty_cycle(self,pulse_duty_cycle):
        self.dev.write(f':PULS:DCYC{self.channel} {pulse_duty_cycle}')
    def set_pulse_width(self,pulse_width):
        self.dev.write(f':PULS:WIDT{self.channel} {pulse_width}')
    

    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'mode','write':self.set_mode,'read':self.get_mode,'type':str,'help':"Set the instrument operation mode (possibilities are: SIN, SQU, RAMP, PULS, NOIS, DC, USER). Note: use USER for arbitrary functions"})
        model.append({'element':'variable','name':'pulse_duty_cycle','write':self.set_pulse_duty_cycle,'unit':'percent','type':float,'help':"Set the duty cycle for pulse mode"})
        model.append({'element':'variable','name':'pulse_width','write':self.set_pulse_width,'unit':'s','type':float,'help':"Set the pulse width for pulse mode"})
        model.append({'element':'variable','name':'amplitude','write':self.amplitude,'unit':'V','type':float,'help':"Set the amplitude"})
        model.append({'element':'variable','name':'offset','write':self.offset,'unit':'V','type':float,'help':"Set the offset"})
        model.append({'element':'variable','name':'frequency','write':self.frequency,'unit':'Hz','type':float,'help':"Set the frequency"})

        return model
    
    
    



# -*- coding: utf-8 -*-
"""
Created on Sun Sep 29 18:26:32 2019

@author: qchat
"""

from PyQt5 import QtCore

class ThreadManager :
    
    """ This class is dedicated to manage the different threads, 
    from their creation, to their deletion them after they have been used """
    
    
    def __init__(self,gui):
        self.gui = gui
        self.threads = {}
        
        
        
    def start(self,item,intType,value=None):
        
        """ This function is called when a new thread is requested, 
        for a particular intType interaction type """
        
        # GUI disabling
        self.gui.tree.setEnabled(False)
        
        # Status writing
        if intType == 'read' : status = f'Reading {item.variable.address()}...'
        elif intType == 'write' : status = f'Writing {item.variable.address()}...'
        elif intType == 'execute' : status = f'Executing {item.action.address()}...'
        self.gui.setStatus(status)

        # Thread configuration
        thread = InteractionThread(item,intType,value)
        tid = id(thread)
        self.threads[tid] = thread
        thread.endSignal.connect(lambda error, x=tid : self.threadFinished(x,error))
        thread.finished.connect(lambda x=tid : self.delete(x))
        
        # Starting thread
        thread.start()
        
        
    def threadFinished(self,tid,error):
        
        """ This function is called when a thread has finished its job, with an error or not 
        It uptates the status bar of the GUI in consequence and enabled back the GUI"""
        
        if error is None : self.gui.clearStatus()
        else : self.gui.setStatus(str(error))
        
        self.gui.tree.setEnabled(True)
        
        
        
    def delete(self,tid):
        
        """ This function is called when a thread is about to be deleted. 
        This removes it from the dictionnary self.threads, for a complete deletion """
        
        self.threads.pop(tid)
        
        
        
        
        
        
class InteractionThread(QtCore.QThread):
    
    """ This class is dedicated to operation interaction with the devices, in a new thread """
    
    endSignal = QtCore.pyqtSignal(object)
    
    
    def __init__(self,item,intType,value):
        QtCore.QThread.__init__(self)
        self.item = item
        self.intType = intType
        self.value = value
        
        
        
    def run(self):
        
        """ Depending on the interaction type requested, this function reads or writes a variable, 
        or execute an action. """
        
        error = None
        
        try :
            if self.intType == 'read' : self.item.variable()
            elif self.intType == 'write' : 
                self.item.variable(self.value)
                if self.item.variable.readable : self.item.variable()
            elif self.intType == 'execute' : 
                if self.value is not None :
                    self.item.action(self.value)
                else :
                    self.item.action()
            
        except Exception as e:
            error = e
        self.endSignal.emit(error)
       
        


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""

import time
import numpy as np

class Driver():
    
    def __init__(self):
        pass
    
            
    def ramp_scanpiezo(self,scanpiezo):
        """Function to scan the piezo with a 50/50 symmetry ramp. 
           Argument scanpiezo is a list of 2 or 3 floats: 1 is the piezo start value, 2 is the piezo stop value, 3 is the total scan period"""
        if (len(scanpiezo) == 3):
            scan_time = float(scanpiezo[2])
            scanpiezo = scanpiezo[:2]
        elif len(scanpiezo)!=2:
            print('Please provide 2 or 3 values for the scan')
            return
        beg,end=[scanpiezo[i] for i in range(len(scanpiezo))]
        ### kernel ###
        step = 1
        end = end+step
        ramp = np.arange(beg,end,step)
        t = time.time();l=[]
        for i in range(len(ramp)):
            self.set_piezo(ramp[i])
            time.sleep(scan_time/len(ramp))
            l.append(time.time()-t)
            print('piezo value: ', ramp[i], '     time elapsed: ', l[i])

    
    def set_piezo(self,piezo):
        self.write(f'SOURce:VOLTage:PIEZo {piezo}')
        #self.read #?? was present
        
    def wavelength(self,lambd):
        self.write(f'SOUR:WAVE {lambd}')
    def set_wavelength_tracking_enabled(self):
        self.write('OUTP:TRAC ON')
    def set_wavelength_tracking_disabled(self):
        self.write('OUTP:TRAC OFF')
    
    def set_scan_start_wavelength(self,val):
        self.write(f'SOURce:WAVE:START {val}')
    def set_scan_stop_wavelength(self,val):
        self.write(f'SOURce:WAVE:STOP {val}')
    def set_scan_forward_velocity(self,val):
        """in nm/s"""
        self.write(f'SOUR:WAVE:SLEW:FORW {val}') 
    def set_scan_backward_velocity(self,val):
        """in nm/s"""
        self.write(f'SOUR:WAVE:SLEW:RET {val}')
    def start_scan_wavelength(self):
        self.write('OUTPut:SCAN:START')


    def idn(self):
        self.write('*IDN?')
        return self.read()


    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'set_piezo','type':float,'write':self.set_piezo,'help':'Set the voltage to apply to the piezo in percent'})
        model.append({'element':'variable','name':'wavelength','write':self.wavelength,'type':float,'help':'Set the wavelength value'})
        model.append({'element':'variable','name':'scan_start_wavelength','type':float,'write':self.set_scan_start_wavelength,'help':'Set start wavelength for wavelength scan'})
        model.append({'element':'variable','name':'scan_stop_wavelength','type':float,'write':self.set_scan_stop_wavelength,'help':'Set stop wavelength for wavelength scan'})
        model.append({'element':'variable','name':'scan_forward_velocity','type':float,'write':self.set_scan_forward_velocity,'help':'Set forward velocity in nm/s for wavelength scan'})
        model.append({'element':'variable','name':'scan_backward_velocity','type':float,'write':self.set_scan_backward_velocity,'help':'Set backward velocity in nm/s for wavelength scan'})
        model.append({'element':'action','name':'start_scan_wavelength','do':self.start_scan_wavelength,'help':'Start scan wavelength'})
        model.append({'element':'action','name':'wavelength_tracking_on','do':self.set_wavelength_tracking_enabled,'help':'Enable wavelength tracking'})
        model.append({'element':'action','name':'wavelength_tracking_off','do':self.set_wavelength_tracking_disabled,'help':'Disable wavelength tracking'})
        return model


#################################################################################
############################## Connections classes ##############################
class Driver_USB(Driver):
    def __init__(self, **kwargs):
        import usb
        import usb.core
        import usb.util
        
        dev = usb.core.find(idVendor=0x104d,idProduct=0x100a)
        dev.reset()
        dev.set_configuration()
        interface = 0
        if dev.is_kernel_driver_active(interface) is True:
            dev.detach_kernel_driver(interface)  # tell the kernel to detach
            usb.util.claim_interface(dev, interface)  # claim the device

        cfg = dev.get_active_configuration()
        intf = cfg[(0,0)]
        self.ep_out = usb.util.find_descriptor(intf,custom_match = lambda e: usb.util.endpoint_direction(e.bEndpointAddress) == usb.util.ENDPOINT_OUT)
        self.ep_in = usb.util.find_descriptor(intf,custom_match = lambda e: usb.util.endpoint_direction(e.bEndpointAddress) == usb.util.ENDPOINT_IN)

        assert self.ep_out is not None
        assert self.ep_in is not None
        
        Driver.__init__(self)
        
    def write(self,query):
        self.string = query + '\r\n'
        self.ep_out.write(self.string)
        
    def read(self):
        rep = self.ep_in.read(64)
        const = ''.join(chr(i) for i in rep)
        const = const[:const.find('\r\n')]
        return const
############################## Connections classes ##############################
#################################################################################




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created on Fri May 17 15:04:04 2019

@author: quentin.chateiller
"""


import sys
import autolab
import argparse


def print_help():
    print()
    print('Usage:')
    print('  autolab [-h] <command> ...')
    print()
    print('Commands:')
    print('  gui                   Start the Graphical User Interface')
    print('  driver                Driver interface')
    print('  device                Device interface')
    print('  doc                   Open the online documentation (readthedocs)')
    print('  report                Open the online report/suggestions webpage (github)')
    print('  infos                 Displays the avalaible drivers and local configurations we ')
    print('  stats                 Manage the data collection state')
    print()
    print('General Options:')
    print('  -h, --help            Show this help message')
    print()


def main() :
    
    args = sys.argv
    
    # No command provided or -h/--help option : print help
    if len(args)==1 or args[1]=='-h' or args[1]=='--help':
        print_help()
        
    else :
        command = args[1]   # first is 'autolab'
        
        # Update the sys.argv for parsers
        args = [f'autolab {command}'] + args[2:]  # first is 'autolab' and second is command 
        sys.argv = args
        
        if command=='doc':   # Open help on read the docs
            autolab.doc()
        elif command=='report':        # Open github report issue webpage
            autolab.report()
        elif command=='gui':           # GUI
            autolab.gui()
        elif command=='infos':           # GUI
            autolab.infos()
        elif command=='stats':
            statistics(args)
        elif command=='driver':
            driver_parser(args)
        elif command=='device':
            device_parser(args)
        else :
            print(f"Command {command} not known. Autolab doesn't have Super Cow Power... yet ^^")
              
            
    sys.exit()
    
####################################################################################
################################## autolab stats ###################################      
def statistics(args_list):
    usage = f"""autolab stats [-h] [options]
    
Management of the anonymous data collection in Autolab.  \n\n
{autolab._stats.get_explanation()}
"""
    parser = argparse.ArgumentParser(add_help=False,usage=usage)
    parser.add_argument("-e", "--enable", action="store_true", dest="enable", help="Enable anonymous data collection." )
    parser.add_argument("-d", "--disable", action="store_true", dest="disable", help="Disable anonymous data collection." )
    parser.add_argument("-q", "--query", action="store_true", dest="query", help="Query anonymous data collection state." )
    parser.add_argument("-h", "--help", action="store_true", dest="help", help="Display this help message." )
    args, unknown = parser.parse_known_args(args_list)
    
    if len(args_list)==1 or args_list[1]=='-h' or args_list[1]=='--help': parser.print_help(); sys.exit()
    assert not(args.enable and args.disable), "You may either enable or disable not both"
    if args.query: print('Stats is ENABLED' if autolab.is_stats_enabled() else 'Stats is DISABLED')
    elif args.enable: autolab.set_stats_enabled(True); print('Turning stats ON permanently')
    elif args.disable: autolab.set_stats_enabled(False); print('Turning stats OFF permanently')
################################## autolab stats ###################################
####################################################################################

####################################################################################
######################### autolab driver/device utilities ##########################
def process_config(args_list):
    
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument("-D", "--driver", type=str, dest="driver", help="Set the nickname or driver to use: 1) uses nickname if it is defined in local_config.ini OR(if it is not) 2) Set the driver name to use." )
    parser.add_argument("-C", "--connection", type=str, dest="connection", help="Set the connection type to use for the connection." )
    parser.add_argument("-A", "--address", type=str, dest="address", help="Set the address to use for the communication." )
    parser.add_argument("-O","--other", nargs='+', dest="other", help="Set other parameters [ports (e.g SOCKET), board_index, slots,...)." )

    args, unknown = parser.parse_known_args(args_list)
    
    config = {}
    if args.connection is not None : config['connection'] = args.connection
    if args.address is not None : config['address'] = args.address
    if args.other is not None : 
        for part in args.other : 
            part = part.replace(' ','')
            config[part.split('=')[0]] = part.split('=')[1]
    
    # Help for autolab driver/device -h/--help and autolab driver/device -h/--help -D driver_name
    if not args.driver or (not args.driver and args_list[1]=='-h' or args_list[1]=='--help'): print_help_parser(parser,args_list); sys.exit()
    if args.driver and (args_list[1]=='-h' or args_list[1]=='--help'): 
        assert args.driver in autolab.list_drivers(), f"Driver {args.driver} not known"
        autolab.config_help(args.driver,_parser=True)
        sys.exit()
    
    return args.driver, config, parser

def print_help_parser(parser,args_list):
    parser.usage = f"""

----------------  General informations:  ----------------

This is a very basic help message for usage of {args_list[0]}. More info can be found on read the doc website (command: autolab doc). To display a more extensive help, please have a look at the section help below.
    
    Usage:   {args_list[0]} -D driver_name -C connection -A address -h

Recquired connection arguments (capital letters):
    -D driver_name: name of the driver to use (e.g.: agilent_33220A). driver_name can be either the driver_name or the defined nickname, as defined by the user in the local_config.ini. See below for accessing the list of available drivers.
    -C connection type to use to communicate with the device (e.g.: VISA, VXI11, SOCKET, TELNET, USB, GPIB, ...). You may access the available connections types with an help (see below helps section).
    -A address: full address to reach the device that depends on the connection type (e.g.: 192.168.0.2  [for VXI11]) and on the way you configured the instrument.
    
    
----------------  Helps (-h option)  ----------------       

Three helps are configured:
    {args_list[0]} -h
    Print this help message.
    
    {args_list[0]} -h -D driver_name
    Print useful informations on the {args_list[0].split(' ')[1]}, including available connection types, etc.
    
    {args_list[0]} -D driver_name -C connection -A address -h
    Full help message about the {args_list[0].split(' ')[1]}. This requires the address to be valid and the {args_list[0].split(' ')[1]} to be instantiated.
    
    
----------------  List of available drivers and local configurations (-D option) ----------------
   
To display the full list of available drivers and local configurations, use:
    autolab infos
    

---------------- Accepted options ----------------"""
    
    parser.print_help()
########################## autolab driver/device utilities ##########################
#####################################################################################


#####################################################################################
################################## autolab driver ###################################
def driver_parser(args_list):
    # Reading of connection information
    driver_name, config, parser = process_config(args_list)
    
    # Reading of methods
    parser.add_argument("-m", "--methods", nargs='+', dest="methods", help="Set the methods to use." )
    parser.add_argument("-h", "--help", dest="help", help="Print this help message." )
    
    # Instantiation of driver.py and driver_utilities.py
    global driver_instance
    driver_instance           = autolab.get_driver(driver_name,**config)
    
    if driver_name in autolab.list_local_configs():        
        # Load config object
        config = dict(autolab.get_local_config(driver_name))
        # Check if driver provided
        assert 'driver' in config.keys(), f"Driver name not found in driver config '{driver_name}'"
        driver_name = config['driver']
        
    driver_utilities          = autolab.load_driver_utilities_lib(driver_name+'_utilities')
    driver_utilities_instance = driver_utilities.Driver_parser(driver_instance,driver_name)
    
    # Add arguments to the existing parser (driver dependant)
    parser = driver_utilities_instance.add_parser_arguments(parser)
    
    # Add help and usage to the parser only if "-h" options requested
    if '-h' in args_list:
        dirver_infos_for_usage = build_driver_infos_for_usage(driver_name,driver_instance)
        parser.usage = driver_utilities_instance.add_parser_usage(dirver_infos_for_usage)
        parser.usage = parser.usage + f"""
    autolab driver -D nickname -m 'some_methods1(arg1,arg2=23)' 'some_methods2(arg1="test")'
    Execute some_methods of the driver. A list of available methods is present at the top of this help along with arguments definition. Note if strings are passed as arguments, mind the usage of ' and " (follow some_methods2 example)."""
        parser.print_help()
        sys.exit()
        
    args = parser.parse_args()

    # Finally, execute functions according to the arguments provided
    driver_utilities_instance.do_something(args)
    
    # Process given methods
    if args.methods:
        global message
        method_list = [method_args[0] for method_args in autolab.get_instance_methods(driver_instance)]
        for method in args.methods:
            message = None
            print(method)
            assert method.split('(')[0] in method_list, f"Method {method} not known or bound. Methods known are: {method_list}"
            print(f'\nExecuting command:  {method}')
            exec(f"message = driver_instance.{method}",globals())
            if message is not None: print(f'Return:  {message}\n')
    
    # Driver closing
    driver_utilities_instance.exit()

def build_driver_infos_for_usage(driver_name,driver_instance):
    driver_lib = autolab.load_driver_lib(driver_name)
    
    # build necessary list
    list_connection_classes = autolab.get_connection_names(driver_lib)
    list_additional_classes = autolab.get_module_names(driver_lib)
    list_instance_methods_and_args = autolab.get_instance_methods(driver_instance)
    
    mess = '\n----------------  Driver informations:  ----------------\n'
    mess += '\n    Available connections types (-C option):\n'
    for connection in list_connection_classes: 
        mess += f'     - {connection}\n'
    mess += '\n'
    mess += '\n    Available additional modules:\n'
    if list_additional_classes==[]:
        mess += f'     {list_additional_classes}'
    else:
        for additional_class in list_additional_classes: 
            mess += f'     - {additional_class}\n'
    mess += '\n'
    mess += '\n    Available methods (with arguments):'
    for method in list_instance_methods_and_args:
        mess += f'\n     - {method[0]}({",".join(method[1])})'
    mess += '\n'
            
    return mess
################################## autolab driver ###################################   
#####################################################################################


#####################################################################################
################################## autolab device ###################################
def device_parser(args_list):
    
    # autolab device -D mydummy -e amplitude -p C:\Users\               GET AND SAVE VARIABLE VALUE
    # autolab device -D mydummy -e something                            EXECUTE ACTION
    # autolab device -D mydummy -e amplitude -v 4                       SET VARIABLE VALUE
    # autolab device -D mydummy -e channel1.amplitude -h                DISPLAY ELEMENT HELP

    # Reading of connection information
    driver_name, config, parser = process_config(args_list)
    
    # Parser configuration
    parser.add_argument('-e', '--element', type=str, dest="element", help='Address of the element to open' )
    parser.add_argument("-v", "--value", type=str, dest="value", help='Value to set')
    parser.add_argument("-p", "--path", type=str, dest="path", help='Path where to save data')
    parser.add_argument("-h", "--help", action='store_true', dest="help", help='Display element help')
    
    # In autolab driver, this is done after the last help request
    args, unknown = parser.parse_known_args(args_list)

    # Instantiation
    instance = autolab.get_device(driver_name,**config)
    element = instance
    
    # Open element
    if args.element is not None :
        for name in args.element.split('.') :
            element = getattr(element,name)
        
    # Execute order
    if args.help is True : element.help()
        
    elif args.path is not None: 
        assert element._element_type == 'variable', f"This element is not a Variable"
        value = element()
        element.save(args.path,value=value)
        
    elif args.value is not None :
        if element._element_type == 'variable' : element(args.value)
        elif element._element_type == 'action' : element(args.value)
        
    else :
        assert element._element_type in ['variable','action'], f"Please provide a Variable or Action element"
        if element._element_type == 'variable' : print(element())
        elif element._element_type == 'action' : element()
    
    instance.close()
################################## autolab device ###################################
#####################################################################################


if __name__ == '__main__' : 
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""    
    
class Driver():

    slot_config = '<MODULE_NAME>'
    
    def __init__(self, **kwargs):
        
        self.write('*CLS')
        
        # Submodules loading
        self.slot_names = {}
        prefix = 'slot'
        for key in kwargs.keys():
            if key.startswith(prefix) and not '_name' in key :
                slot_num = key[len(prefix):]
                module_name = kwargs[key].strip()
                module_class = globals()[f'Module_{module_name}']
                if f'{key}_name' in kwargs.keys() : name = kwargs[f'{key}_name']
                else : name = f'{key}_{module_name}'
                setattr(self,name,module_class(self,slot_num))
                self.slot_names[slot_num] = name

    def get_id(self):
        return self.write('*IDN?')
    
    def get_driver_model(self):
        
        model = []
        for name in self.slot_names :
            model.append({'element':'module','name':name,'object':getattr(self,name)})
        return model
    
    
    
#################################################################################
############################## Connections classes ##############################
class Driver_TELNET(Driver):
    
    def __init__(self, address='192.168.0.12', **kwargs):
        from telnetlib import Telnet
                
        # Instantiation
        self.controller = Telnet(address,5024)
        while True : 
            ans = self.read(timeout=0.1)
            if ans is not None and 'Connected' in ans :
                break

        
        Driver.__init__(self,**kwargs)
        
        
    def write(self,command):
        try : self.controller.write(f'{command}\r\n'.encode())
        except : pass
        return self.read()
        
        
    def read(self,timeout=1):
        try :
            ans = self.controller.read_until('READY>'.encode(),timeout=timeout)
            ans = ans.decode().replace('READY>','').strip() 
            assert ans != ''
            return ans
        except :
            pass
        
    def close(self):
        try : self.controller.close()
        except : pass
    
############################## Connections classes ##############################
#################################################################################
        
    
    
class Module_IQS9100B():
    
    
    def __init__(self,dev,slot):
        
        self.dev = dev
        self.SLOT = slot
        self.prefix = f'LINS1{self.SLOT}:'
        
        # Initialisation
        self.dev.write(self.prefix+f'STAT?')
        self.dev.write('*OPC?')
        
    def safe_state(self):
        self.set_shutter(True)
        if self.is_shuttered() is True :
            return True
   

    def get_id(self):
        return self.dev.write(self.prefix+f'SNUM?')
        
        
    def set_route(self,route_id):
        curr_route = self.get_route()
        if curr_route != route_id :
            self.dev.write(self.prefix+f"ROUT1:SCAN {int(route_id)}")
            self.dev.write(self.prefix+f'ROUT1:SCAN:ADJ')
            self.dev.write('*OPC?')

    def get_route(self):
        ans=self.dev.write(self.prefix+f'ROUT1:SCAN?')
        return int(ans)



    def is_shuttered(self):
        ans=self.dev.write(self.prefix+f'ROUT1:OPEN:STAT?')
        return not bool(int(ans))
        
    def set_shuttered(self,value):
        assert isinstance(value,bool)
        if value is False :
            self.dev.write(self.prefix+f"ROUT1:OPEN")
        else :
            self.dev.write(self.prefix+f"ROUT1:CLOS")
        self.dev.write('*OPC?')
        
    def get_driver_model(self):
        
        model = []
        model.append({'element':'variable','name':'route','type':int,'read':self.get_route,'write':self.set_route,'help':'Current route of the switch'})
        model.append({'element':'variable','name':'shuttered','type':bool,'read':self.is_shuttered,'write':self.set_shuttered,'help':'State of the shutter'})
        model.append({'element':'action','name':'safe_state','do':self.safe_state,'help':'Set the shutter'})
        return model
        



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""


class Driver():

    slot_config = '<MODULE_NAME>'
    
    def __init__(self,**kwargs):
        
        # Submodules loading
        self.slot_names = {}
        prefix = 'slot'
        for key in kwargs.keys():
            if key.startswith(prefix) and not '_name' in key :
                slot_num = key[len(prefix):]
                module_name = kwargs[key].strip()
                module_class = globals()[f'Module_{module_name}']
                if f'{key}_name' in kwargs.keys() : name = kwargs[f'{key}_name']
                else : name = f'{key}_{module_name}'
                setattr(self,name,module_class(self,slot_num))
                self.slot_names[slot_num] = name
                

        
    def get_id(self):
        return self.write('*IDN?')
    
    
    def get_driver_model(self):
        model = []
        for name in self.slot_names.values() :
            model.append({'element':'module','name':name,'object':getattr(self,name)})
        return model
    
#################################################################################
############################## Connections classes ##############################
class Driver_TELNET(Driver):
    
    def __init__(self, address='192.168.0.1', **kwargs):
        from telnetlib import Telnet
        
        # Instantiation
        self.controller = Telnet(address,5024)
        while True : 
            ans = self.read(timeout=0.1)
            if ans is not None and 'Connected' in ans :
                break

        Driver.__init__(self, **kwargs)
        
    def write(self,command):
        self.controller.write(f'{command}\r\n'.encode())
        return self.read()
        
        
    def read(self,timeout=1):
        try :
            ans = self.controller.read_until('READY>'.encode(),timeout=timeout)
            ans = ans.decode().replace('READY>','').strip() 
            assert ans != ''
            return ans
        except :
            pass
        
    def close(self):
        try : self.controller.close()
        except : pass
    
############################## Connections classes ##############################
#################################################################################    
    
    
    
class Module_FTB1750():
    
    def __init__(self,dev,slot):
        
        self.dev = dev
        self.SLOT = slot
        
        # Initialisation
        self.dev.write(f"LINS1:UNIT{self.SLOT}:POW W")                # Unité = Watts
        self.dev.write(f"LINS1:SENS{self.SLOT}:POW:RANG:AUTO 1")      # Ajuster la gamme de mesure automatiquement
        self.dev.write(f"LINS1:SENS{self.SLOT}:POW:REF:STAT 0")       # Set Absolute power measurment mode (dBm or W)
        self.dev.write('*OPC?')
    



    
    def set_averaging_state(self,state):
        assert isinstance(state,bool)
        self.dev.write(f"LINS1:SENS{self.SLOT}:AVER:STAT {int(state)}")
        self.dev.write('*OPC?')
    
    def get_averaging_state(self):
        ans = self.dev.write(f"LINS1:SENS{self.SLOT}:AVER:STAT?")
        return bool(int(ans))



    
    
    def get_buffer_size(self):
        ans = self.dev.write(f"LINS1:SENS{self.SLOT}:AVER:COUN?")
        return int(ans)
    
    def set_buffer_size(self, value):
        self.dev.write(f"LINS1:SENS{self.SLOT}:AVER:COUN {value}")
        self.dev.write('*OPC?')


 
       

    
    def get_power(self):
        ans = self.dev.write(f"LINS1:READ{self.SLOT}:SCAL:POW:DC?")
        return float(ans)
    

    
        
    
    
    def set_wavelength(self,wavelength):
        self.dev.write(f"LINS1:SENS{self.SLOT}:POW:WAV {wavelength} nm")
        self.dev.write('*OPC?')
    
    def get_wavelength(self):
        ans = self.dev.write(f"LINS1:SENS{self.SLOT}:POW:WAV?")
        return float(ans)*1e9
    
    
    
    
    
    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'averaging','type':bool,'read':self.get_averaging_state,'write':self.set_averaging_state,'help':'Average or not the measure'})
        model.append({'element':'variable','name':'buffer_size','type':int,'read':self.get_buffer_size,'write':self.set_buffer_size,'help':'Buffer size for the average'})
        model.append({'element':'variable','name':'wavelength','type':float,'unit':'nm','read':self.get_wavelength,'write':self.set_wavelength,'help':'Wavelength of the measure'})
        model.append({'element':'variable','name':'power','type':float,'unit':'W','read':self.get_power,'help':'Current power'})
        return model
        



# -*- coding: utf-8 -*-
"""
Created on Fri Sep 20 17:11:57 2019

@author: qchat
"""

import webbrowser

def report():
    webbrowser.open('https://github.com/qcha41/autolab/issues')
    
def doc():
    webbrowser.open('https://autolab.readthedocs.io')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified): Yenista Tunics.
- 
"""


class Driver():
    
    def __init__(self):
        self.write('MW')
        
    def wait(self):
        self.get_id() # Not fantastic but the tunics API is really basic
             
    def get_id(self):
        return self.query('*IDN?')
    
    def set_frequency(self,value):
        self.write(f"F={value}")
        self.wait()
        
    def get_frequency(self):
        return float(self.query("F?").split('=')[1])

    def set_wavelength(self,value):
        self.write(f"L={value}")
        self.wait()
        
    def get_wavelength(self):
        return float(self.query("L?").split('=')[1])
    
    def set_power(self,value):
        self.write(f"P={float(value)}")
        if value == 0 : self.set_output(False)
        else : 
            if self.get_output() is False : self.set_output(True)
        self.wait()
        
    def get_power(self):
        ans=self.query("P?")
        if ans == 'DISABLED' : return 0
        else : return float(ans.split('=')[1])
    
    def set_intensity(self,value):
        self.write(f"I={float(value)}")
        if value == 0 : self.set_output(False)
        else :
            if self.get_output() is False : self.set_output(True)
        self.wait()
        
    def get_intensity(self):
        ans=self.query("I?")
        if ans == 'DISABLED' : return 0
        else : return float(ans.split('=')[1])
        
    def set_output(self,state):
        if state is True : self.write("ENABLE")
        else : self.write("DISABLE")
        self.wait()
        
    def get_output(self):
        ans = self.query("P?")
        if ans == 'DISABLED' : return False
        else : return True
        
        
    def get_motor_speed(self):
        return float(self.query("MOTOR_SPEED?"))
 
    
    def set_motor_speed(self,value):  # from 1 to 100 nm/s
        self.write("MOTOR_SPEED={float(value)}")
        self.wait()


    def get_driver_model(self):
        
        config = []

        config.append({'element':'variable','name':'wavelength','unit':'nm','type':float,
                       'read':self.get_wavelength,'write':self.set_wavelength})
    
        config.append({'element':'variable','name':'frequency','unit':'GHz','type':float,
                       'read':self.get_frequency,'write':self.set_frequency})

        config.append({'element':'variable','name':'power','unit':'mW','type':float,
                       'read':self.get_power,'write':self.set_power})

        config.append({'element':'variable','name':'intensity','unit':'mA','type':float,
                       'read':self.get_intensity,'write':self.set_intensity})
    
        config.append({'element':'variable','name':'output','type':bool,
                       'read':self.get_output,'write':self.set_output})

        config.append({'element':'variable','name':'motor_speed','unit':'nm/s','type':float,
                       'read':self.get_motor_speed,'write':self.set_motor_speed})
    
        return config



#################################################################################
############################## Connections classes ##############################
class Driver_VISA(Driver):
    def __init__(self, address='GPIB0::2::INSTR', **kwargs):
        import visa    
        
        self.TIMEOUT = 15000 #ms
        
        rm = visa.ResourceManager()
        self.controller = rm.open_resource(address)
        self.controller.timeout = self.TIMEOUT
        
        Driver.__init__(self)
    
    def close(self):
        try : self.controller.close()
        except : pass

    def query(self,command):
        result = self.controller.query(command)
        result = result.strip('\n')
        return result
    
    def write(self,command):
        self.controller.write(command)
    
    def read(self):
        return self.controller.read()


class Driver_GPIB(Driver):
    def __init__(self,address=23,board_index=0,**kwargs):
        import Gpib
        
        self.inst = Gpib.Gpib(int(board_index),int(address))
        Driver.__init__(self)
    
    def query(self,query):
        self.write(query)
        return self.read()
    def write(self,query):
        self.inst.write(query)
    def read(self,length=1000000000):
        return self.inst.read().decode().strip('\n')
    def close(self):
        """WARNING: GPIB closing is automatic at sys.exit() doing it twice results in a gpib error"""
        #Gpib.gpib.close(self.inst.id)
        pass
############################## Connections classes ##############################
#################################################################################
        




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Function generator'

class Driver_parser():
    def __init__(self, Instance, name, **kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}

----------------  Examples:  ----------------

usage:    autolab driver [options] args
            
    autolab driver -D {self.name} -C USB -a 0.5 -f 500 -c 1,2
    load {self.name} driver using pyusb and set the amplitude to 0.5V and frequency to 500Hz to channels 1 and 2.
    
    autolab driver -D nickname -a 0.5 -f 500 -c 1
    same as before but using the device nickname as defined in local_config.ini, only acting on channel 1
            """
        return usage
    
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        parser.add_argument("-c", "--channels", type=str, dest="channels", default=None, help="Set the channels to act on/acquire from." )
        parser.add_argument("-a", "--amplitude", type=str, dest="amplitude", default=None, help="Set the amplitude." )
        parser.add_argument("-f", "--frequency", type=str, dest="frequency", default=None, help="Set the frequency." )
        parser.add_argument("-p", "--period", type=str, dest="period", default=None, help="Set the period." )
        
        return parser

    def do_something(self,args):
        if args.channels:
            for chan in args.channels.split(','):
                if args.amplitude:
                    getattr(getattr(self.Instance,f'channel{chan}'),'amplitude')(args.amplitude)
                if args.frequency:
                    getattr(getattr(self.Instance,f'channel{chan}'),'frequency')(args.frequency)
                if args.period:
                    getattr(getattr(self.Instance,f'channel{chan}'),'period')(args.period)


    def exit(self):
        self.Instance.close()



# -*- coding: utf-8 -*-
"""
Created on Mon Nov 18 14:53:10 2019

@author: qchat
"""

import configparser
import autolab
import os
import shutil

def check():
    
    """ This function create the default autolab local directory """
        
    # LOCAL DIRECTORY
    if os.path.exists(autolab.paths.USER_FOLDER) is False:
        os.mkdir(autolab.paths.USER_FOLDER)
        print(f'The local directory of AUTOLAB has been created: {autolab.paths.USER_FOLDER}')

    # LOCAL CONFIGURATION FILE
    if os.path.exists(autolab.paths.LOCAL_CONFIG) is False : 
        shutil.copyfile(autolab.paths.LOCAL_CONFIG_TEMPLATE,autolab.paths.LOCAL_CONFIG)
        print(f'The configuration file local_config.ini has been created: {autolab.paths.LOCAL_CONFIG}')
    
    # lOCAL CUSTOM DRIVER FOLDER
    if os.path.exists(autolab.paths.DRIVER_SOURCES['local']) is False :
        os.mkdir(autolab.paths.DRIVER_SOURCES['local'])
        print(f'The local driver directory has been created: {autolab.paths.DRIVER_SOURCES["local"]}')
    
    # AUTOLAB CONFIGURATION FILE
    if os.path.exists(autolab.paths.AUTOLAB_CONFIG) is False :
        save(configparser.ConfigParser())
        print(f'The configuration file autolab_config.ini has been created: {autolab.paths.AUTOLAB_CONFIG}')
    
    config = load()
    
    # stats
    if 'stats' not in config.sections() or 'enabled' not in config['stats'].keys() :
        
        ans = input(f'{autolab._stats.get_explanation()} Do you agree? [default:yes] > ')
        if ans.strip().lower() == 'no' :
            print('This feature has been disabled. You can enable it back with the function autolab.set_stats_enabled(True).')
            config['stats'] = {'enabled': '0'}
        else :
            print('Thank you !')
            config['stats'] = {'enabled': '1'}
    
        save(config)
        
        

def set_value(header,parameter,value):
    
    ''' This function set the value of the given parameter of the given header in the autolab configuration file'''
    
    config = load()
    assert header in config.sections(), f"Header '{header}' not found in the autolab configuration file."
    assert parameter in config[header].keys(), f"Parameter '{parameter}' not found in header {header}."
    config[header][parameter] = str(value)
    save(config)



def get_value(header,parameter):
    
    ''' This function set the value of the given parameter of the given header in the autolab configuration file'''
     
    config = load()
    assert header in config.sections(), f"Header '{header}' not found in the autolab configuration file."
    assert parameter in config[header].keys(), f"Parameter '{parameter}' not found in header {header}."
    return config[header][parameter]
   
    

def save(config):
    
    """ This function saves the given config parser in the autolab configuration file """
    
    with open(autolab.paths.AUTOLAB_CONFIG, 'w') as file:
        config.write(file)
    
    

def load():
    
    """ This function loads the autolab configuration file in a config parser """
        
    config = configparser.ConfigParser()
    config.read(autolab.paths.AUTOLAB_CONFIG)
    
    return config




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

category = 'Function generator'

class Driver_parser():
    def __init__(self, Instance, name,**kwargs):
        self.name     = name
        self.Instance = Instance
        
        
    def add_parser_usage(self,message):
        """Usage to be used by the parser"""
        usage = f"""
{message}

----------------  Examples:  ----------------

usage:    autolab driver [options] args

    autolab driver -D {self.name} -A TCPIP::192.168.0.3::INSTR -C VISA -a 1 -o 1 -f 50KHZ -c 1,2
    set the frequency to 50 kHz, the amplitude to 1V, the offset to 1V for both channel 1 and 2

    autolab driver -D nickname -p w10NS 1
    set pulse mode to channel 1 with pulse width of 10NS (see below -p option help for further assistance), using the device nickname as defined in local_config.ini

    autolab driver -D nickname -p d10 2
    set pulse mode to channel 2 with duty cycle of 10 percent, using the device nickname as defined in local_config.ini
            """
        return usage
        
    def add_parser_arguments(self,parser):
        """Add arguments to the parser passed as input"""
        parser.add_argument("-c", "--channels", type=str, dest="channels", default=None, help="Set the channels to act on/acquire from." )
        parser.add_argument("-o", "--offset", type=str, dest="offset", default=None, help="Set the offset value in Volts." )
        parser.add_argument("-d", "--dc_mode", type=str, dest="dc_mode", default=None, help="Set dc mode on and set offset." )
        parser.add_argument("-p", "--pulsemode", type=str, dest="pulsemode", default=None, help="Set pulse mode and use argument as either the duty cycle or the pulse width depending on the first letter 'd' or 'w' (see examples). The duty cycle unit is percent. The pulse width unit is second; you may specify it as 'w10MS'(for microseconds) 'w10NS'(for nanoseconds); note that it is also possible to write 'w10e-9'." )
        parser.add_argument("-a", "--amplitude", type=str, dest="amplitude", default=None, help="Set the amplitude in Volts." )
        parser.add_argument("-f", "--frequency", type=str, dest="frequency", default=None, help="Set the frequency in Hz. Values may be 50000 or 50KHZ" )
        
        return parser

    def do_something(self,args):
        if args.channels:
            for chan in args.channels.split(','):
                if args.amplitude:
                    getattr(getattr(self.Instance,f'channel{chan}'),'amplitude')(args.amplitude)
                if args.offset:
                    getattr(getattr(self.Instance,f'channel{chan}'),'offset')(args.offset)
                if args.frequency:
                    getattr(getattr(self.Instance,f'channel{chan}'),'frequency')(args.frequency)
                if args.dc_mode:
                    import time
                    getattr(getattr(self.Instance,f'channel{chan}'),'set_mode')('DC')
                    time.sleep(0.03)     # so ugly.., makes a difference at display
                    getattr(getattr(self.Instance,f'channel{chan}'),'offset')(args.dc_mode)
                if args.pulsemode:
                    if args.pulsemode[0]=="d": 
                        getattr(getattr(self.Instance,f'channel{chan}'),'set_mode')('PULS')
                        getattr(getattr(self.Instance,f'channel{chan}'),'set_pulse_duty_cycle')(args.pulsemode[1:])
                    elif args.pulsemode[0]=="w":
                        getattr(getattr(self.Instance,f'channel{chan}'),'set_mode')('PULS')
                        getattr(getattr(self.Instance,f'channel{chan}'),'set_pulse_width')(args.pulsemode[1:])
                    else: print("pulse mode argument must start with either 'd' or 'w'")
                    

    def exit(self):
        self.Instance.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Supported instruments (identified):
- 
"""

import pandas as pd
import numpy as np





class Driver():    
    
    def __init__(self):
                        
        self.min_counts_allowed=5000
        self.max_counts_allowed=61000
        
        # Defaults
        self.auto_background_removal = False
        self.auto_exposure_time = True
		
        self.data = {'exposure_time':None,'spectrum':None}
        self.write('Initialize')
                
        
        
    def is_connected(self):
        try :
            return bool(int(self.query('STATE?')))
        except :
            return False


    

        
        
    def get_exposure_time(self):
        if self.data['exposure_time'] is None :
            self.data['exposure_time'] = float(self.query('EXPTIME?'))
        return self.data['exposure_time']

    def set_exposure_time(self,value):
        value=float(value)
        self.write(f'EXPTIME={value}')
        self.data['exposure_time'] = float(self.query('EXPTIME?'))
        
        
        
        
        
    
    def is_auto_exposure_time_enabled(self):
        return self.auto_exposure_time
    
    def set_auto_exposure_time_enabled(self,value):
        assert isinstance(value,bool)
        self.auto_exposure_time = value
        
        
        
        
        
        
    def get_spectrum(self):
        if self.data['spectrum'] is None :
            self.acquire_spectrum()
        return self.data['spectrum']
    
    
    def acquire_spectrum(self):
            
        if self.is_auto_exposure_time_enabled() :
            
            while True :
                
                # Mesure spectre
                spectrum = pd.read_json(self.query('SPECTRUM?'))
        
                # Récupération des données
                max_value=max(spectrum['counts'])
                
                # Reduction du temps d'exposition
                if max_value>self.max_counts_allowed : 
                    exposure_time_save = self.get_exposure_time()
                    self.set_exposure_time(self.get_exposure_time()/10)
                    if self.get_exposure_time() == exposure_time_save :
                        break
                
                # Augmentation du temps d'exposition
                elif max_value<self.min_counts_allowed : 
                    exposure_time_save = self.get_exposure_time()
                    self.set_exposure_time(self.get_exposure_time()*self.max_counts_allowed/max_value*0.9)
                    if self.get_exposure_time() == exposure_time_save :
                        break
                    
                else :
                    break
                
            self.data['spectrum'] = spectrum
            
        else :
            
            self.data['spectrum'] = pd.read_json(self.query('SPECTRUM?'))
            
		
        self.data['spectrum'].sort_index(inplace=True)
        
        if self.is_auto_background_removal_enabled():
            self.data['spectrum']['CountsWithoutBackground'] =(self.data['spectrum'].counts - self.getBackground())
            self.data['spectrum']['power']=self.data['spectrum']['CountsWithoutBackground']/self.get_exposure_time()
        else:
            self.data['spectrum']['power']=self.data['spectrum']['counts']/self.get_exposure_time()

            
            
    def getBackground(self):
        if self.data['spectrum'] is None :
            self.acquire_spectrum()
            self.data['spectrum'].sort_index(inplace=True)

        total_rows = len(self.data['spectrum'].index)
        mean_background_blue_wl = self.data['spectrum'].counts.head(int(np.floor(total_rows/10))).mean()
        mean_background_red_wl = self.data['spectrum'].counts.tail(int(np.floor(total_rows/10))).mean()
        mean_background = (mean_background_red_wl + mean_background_blue_wl)/2
        return mean_background
    
    
    def is_auto_background_removal_enabled(self):
        return self.auto_background_removal
    
    
    
    def set_auto_background_removal_enabled(self,value):
        assert isinstance(value,bool)
        self.auto_background_removal = value
    
    
    def get_temperature(self):
        return float(self.query('TEMP?'))
    
    
    
    
    

        
        
    def get_main_peak_wavelength(self):
        if self.data['spectrum'] is None :
            self.acquire_spectrum()
        power = self.data['spectrum']['power']
        idx = (power-power.max()).abs().idxmin()
        return self.data['spectrum']['wavelength'].loc[idx]
        
    
    
    
    
    def get_max_power(self):
        if self.data['spectrum'] is None :
            self.acquire_spectrum()
        return self.data['spectrum']['power'].max()
    
    
    
    
    
    def get_integrated_power(self):
        if self.data['spectrum'] is None :
            self.acquire_spectrum()
        return np.trapz(self.data['spectrum']['power'],self.data['spectrum']['wavelength'])
    
    
    
    
    def get_main_peak_fwhm(self):
        
        if self.data['spectrum'] is None :
            self.acquire_spectrum()
        
        power = self.data['spectrum']['power']
        
        # Recherche du max        
        idxMax=(power-power.max()).abs().idxmin()
        halfPower=self.get_max_power()/2
                      
        # Recherche autour du pic
        for i in np.arange(idxMax,min(power.index.values),-1):
            if power.loc[i] - halfPower<0 :
                fwhm_idxMin=i
                break
        for i in np.arange(idxMax,max(power.index.values),1):
            if power.loc[i] - halfPower <0 :
                fwhm_idxMax=i
                break
                
        wavelength = self.data['spectrum']['wavelength']
        return wavelength.loc[fwhm_idxMax] - wavelength.loc[fwhm_idxMin]
    
    
    
    def get_driver_model(self):
        model = []
        model.append({'element':'variable','name':'exposure_time','unit':'s','type':float,'read':self.get_exposure_time,'write':self.set_exposure_time,'help':'Exposure time of the camera'})
        model.append({'element':'variable','name':'auto_exposure_time','type':bool,'read':self.is_auto_exposure_time_enabled,'write':self.set_auto_exposure_time_enabled,'help':'Enable or not the auto exposure time mode'})
        model.append({'element':'variable','name':'auto_background_removal','type':bool,'read':self.is_auto_background_removal_enabled,'write':self.set_auto_background_removal_enabled,'help':'Enable or not the auto background removal mode'})
        model.append({'element':'variable','name':'spectrum','read':self.get_spectrum,'type':pd.DataFrame,'help':'Spectrum acquired'})
        model.append({'element':'variable','name':'temperature','type':float,'unit':'°C','read':self.get_temperature,'help':'Temperature of the camera'})
        model.append({'element':'variable','name':'main_peak_wavelength','type':float,'unit':'nm','read':self.get_main_peak_wavelength,'help':'Wavelength of the main peak in the spectrum'})
        model.append({'element':'variable','name':'main_peak_peak_fwhm','type':float,'unit':'nm','read':self.get_main_peak_fwhm,'help':'FWHM of the main peak in the spectrum'})
        model.append({'element':'variable','name':'max_power','type':float,'read':self.get_max_power,'help':'Maximum power of the main peak in the spectrum'})
        model.append({'element':'variable','name':'integrated_power','type':float,'read':self.get_integrated_power,'help':'Integrated power of the spectrum'})
        model.append({'element':'action','name':'acquire','do':self.acquire_spectrum,'help':'Acquire a spectrum'})
        return model
    
    
    
#################################################################################
############################## Connections classes ##############################
class Driver_SOCKET(Driver) :
    
    def __init__(self,address='192.168.0.8',**kwargs):
        
        import socket 
        
        self.ADDRESS = address
        self.PORT = 5005
        self.BUFFER_SIZE = 40000
        
        self.controller = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.controller.connect((self.ADDRESS,int(self.PORT)))   
        
        Driver.__init__(self)
        
    def write(self,command):
        self.controller.send(command.encode())
        self.controller.recv(self.BUFFER_SIZE)
        
    def query(self,command):
        self.controller.send(command.encode())
        data = self.controller.recv(self.BUFFER_SIZE)
        return data.decode()
        
    def close(self):
        try :
            self.controller.close()
        except :
            pass
        self.controller = None



class Driver_LOCAL(Driver) : #not used

    def __init__(self,**kwargs):
        
        from winspec_utilities.winspec_gui_driver import Winspec
        self.controller = Winspec()

        Driver.__init__(self)
        
              
    def write(self,command):
        self.controller.command(command)
        
    def query(self,command):
        return self.controller.command(command)
        
    def close(self):
        self.controller = None
############################## Connections classes ##############################
#################################################################################



